/*
* Shutter - Friend List Component
*
* Copyright (C) 2004 luma <stubma@163.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
package edu.tsinghua.swt.internal.events;

import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.widgets.Text;

import edu.tsinghua.swt.events.CoolButtonEvent;
import edu.tsinghua.swt.events.CoolButtonListener;
import edu.tsinghua.swt.widgets.CoolButton;

/**
 * Process editor key event, if CR is entered, end editing and set new text
 * 
 * @author 
 */
public class EditorKeyListener extends KeyAdapter {
	// CoolButton object
	private CoolButton ancestor;
	// shutter label event listeners
	private List listeners;
	
	/**
	 * constructor
	 * @param ancestor
	 * @param listeners
	 */
	public EditorKeyListener(CoolButton ancestor, List listeners) {
		this.ancestor = ancestor;
		this.listeners = listeners;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.KeyListener#keyPressed(org.eclipse.swt.events.KeyEvent)
	 */
	public void keyPressed(KeyEvent e) {
		if(e.character==SWT.CR) {
			// new a event object
			CoolButtonEvent event = new CoolButtonEvent(ancestor);
			event.oldText = ancestor.getText();
			if(event.oldText == null)
			    event.oldText = "";
			
			// set new text
			Text editor = (Text)e.getSource();
			ancestor.setText(editor.getText());
			editor.setVisible(false);
			
			// if text changed, fire event
			if(!event.oldText.equals(ancestor.getText())) {
				event.newText = ancestor.getText();
				int size = listeners.size();
				for(int i = 0; i < size; i++)
					((CoolButtonListener)listeners.get(i)).textChanged(event);
			}
		}
	}
}
