/*
* Shutter - Friend List Component
*
* Copyright (C) 2004 luma <stubma@163.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
package edu.tsinghua.swt.layout;

import org.eclipse.swt.widgets.Control;

/**
 * Sort layout data
 * 
 * @author 
 */
public class SortData {
	// layout index when layout is performing
	//    if it's -1, go to the last
	public int index;
	// sort field
	public Object key;
	// adhere to which control
	public Control control;
	// layout data self-explained variable
	public int verticalAlignment;
	public int horizontalAlignment;
	public int horizontalIndent;
	public int verticalIndent;
	public int widthHint;
	public int heightHint;
	public boolean grabExcessHorizontalSpace;
	public boolean grabExcessVerticalSpace;
	public int adhereTo;
	public boolean beforeAdheredControl;
	// ListData constant
	public static final int BEGINNING = 0x1;
	public static final int CENTER = 0x2;
	public static final int END = 0x3;
	public static final int FILL = 0x4;
	public static final int INDEX = 0x8;
	public static final int CONTROL = 0x10;
	public static final int NONE = 0x20;
	public static final int LAST = Integer.MAX_VALUE;
	public static final int FIRST = 0;
	public static final int VERTICAL_ALIGN_BEGINNING = 0x2;
	public static final int VERTICAL_ALIGN_CENTER = 0x4;
	public static final int VERTICAL_ALIGN_END = 0x8;
	public static final int VERTICAL_ALIGN_FILL = 0x10;
	public static final int HORIZONTAL_ALIGN_BEGINNING = 0x20;
	public static final int HORIZONTAL_ALIGN_CENTER = 0x40;
	public static final int HORIZONTAL_ALIGN_END = 0x80;
	public static final int HORIZONTAL_ALIGN_FILL = 0x100;
	public static final int GRAB_HORIZONTAL = 0x200;
	public static final int GRAB_VERTICAL = 0x400;
	public static final int FILL_VERTICAL = 0x410;
	public static final int FILL_HORIZONTAL = 0x300;
	public static final int FILL_BOTH = 0x710;
	// control index of parent's children array 
	int childIndex;
	
	/**
	 * constructor
	 */
	public SortData()
	{
		this(VERTICAL_ALIGN_BEGINNING | HORIZONTAL_ALIGN_BEGINNING);
	}
	
	/**
	 * constructor
	 * @param key
	 */
	public SortData(Object key) {
		this(VERTICAL_ALIGN_BEGINNING | HORIZONTAL_ALIGN_BEGINNING, key);
	}
	
	/**
	 * constructor
	 * @param style control style, such as alignment
	 */
	public SortData(int style) {
		this(style, LAST, null);
	}
	
	/**
	 * constructor
	 * @param style control style, such as alignment
	 * @param key sort field
	 */
	public SortData(int style, Object key) {
		this(style, LAST, key);
	}
	
	/**
	 * constructor
	 * @param style control style, such as alignment
	 * @param index the index
	 * @param key key use to sort
	 */
	public SortData(int style, int index, Object key) {
		this(style, index, key, NONE, null, false);		
	}
	
	/**
	 * constructor
	 * @param style control style, such as alignment
	 * @param index the index
	 * @param adhere true if we need keep index not changed
	 */
	public SortData(int style, int index, boolean adhere) {
		this(style, index, null, NONE, null, false);
		if(adhere) this.adhereTo = INDEX;
	}
	
	/**
	 * constructor
	 * @param index the index
	 * @param adhere adhere to index or not
	 */
	public SortData(int index, boolean adhere) {
		this(VERTICAL_ALIGN_BEGINNING | HORIZONTAL_ALIGN_BEGINNING, index, adhere);
	}
	
	/**
	 * constructor
	 * @param style control style, such as alignment
	 * @param index the index
	 */
	public SortData(int style, int index) {
		this(style, index, false);
	}
	
	/**
	 * constructor
	 * @param style control style, such as alignment
	 * @param control control adhered to
	 * @param beforeAdheredControl true if place it before the control
	 */
	public SortData(int style, Control control, boolean beforeAdheredControl) {
		this(style, LAST, null, CONTROL, control, beforeAdheredControl);
	}
	
	/**
	 * constructor
	 * @param control control adhered to
	 * @param beforeAdheredControl true if place it before the control
	 */
	public SortData(Control control, boolean beforeAdheredControl) {
		this(VERTICAL_ALIGN_BEGINNING | HORIZONTAL_ALIGN_BEGINNING, control, beforeAdheredControl);
	}
	
	/**
	 * constructor
	 * @param control control adhered to
	 */
	public SortData(Control control) {
		this(VERTICAL_ALIGN_BEGINNING | HORIZONTAL_ALIGN_BEGINNING, control, false);
	}
	
	/**
	 * constructor
	 * @param style control style, such as alignment
	 * @param control control adhered to
	 */
	public SortData(int style, Control control) {
		this(style, control, false);		
	}
	
	/**
	 * constructor
	 * @param style control style, such as alignment
	 * @param adhereTo adhereTo what, index, control or not fixed
	 * @param index the index control adhere to
	 * @param beforeAdheredControl adhere to the front or the rear of the control
	 */
	public SortData(int style, int adhereTo, int index, boolean beforeAdheredControl) {
		this(style, index, null, adhereTo, null, beforeAdheredControl);
	}
	
	// private constructor
	//    only serve as a variable initializer
	private SortData(int style, int index, Object key, int adhereTo, Control control, boolean beforeAdheredControl)
	{
		this.index = index;
		this.key = key;
		this.adhereTo = adhereTo;
		this.control = control;
		this.beforeAdheredControl = beforeAdheredControl;
		verticalAlignment = BEGINNING;
		horizontalAlignment = BEGINNING;
		widthHint = -1;
		heightHint = -1;
		horizontalIndent = 0;
		verticalIndent = 0;
		grabExcessHorizontalSpace = false;
		grabExcessVerticalSpace = false;
		if((style & 2) != 0)
			verticalAlignment = 1;
		if((style & 4) != 0)
			verticalAlignment = 2;
		if((style & 0x10) != 0)
			verticalAlignment = 4;
		if((style & 8) != 0)
			verticalAlignment = 3;
		if((style & 0x20) != 0)
			horizontalAlignment = 1;
		if((style & 0x40) != 0)
			horizontalAlignment = 2;
		if((style & 0x100) != 0)
			horizontalAlignment = 4;
		if((style & 0x80) != 0)
			horizontalAlignment = 3;
		if((style & 0x200) != 0)
			grabExcessHorizontalSpace = true;
		else
			grabExcessHorizontalSpace = false;
		if((style & 0x400) != 0)
			grabExcessVerticalSpace = true;
		else
			grabExcessVerticalSpace = false;
	}
}
