/*
* Shutter - Friend List Component
*
* Copyright (C) 2004 luma <stubma@163.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
package edu.tsinghua.swt.models;

import java.util.ArrayList;
import java.util.Hashtable;

import edu.tsinghua.swt.events.PropertyEvent;
import edu.tsinghua.swt.events.PropertyListener;

/**
 * Default GeneralItem implementation, intended to subclassed
 * @author luma@smth
 */
public abstract class AbstractNode implements INode {
	// property hash
	protected Hashtable properties;
	// property listeners
	private ArrayList listeners;

	/**
	 * constructor
	 */
	public AbstractNode() {
		properties = new Hashtable();
		listeners = new ArrayList();
	}
	
	/**
	 * һԣµԺ͵ǰǷȣȾֱӷأ
	 * ȣԱ޸¼ԭڣԱ¼
	 * Object͵ԣûҪʵequalsԱжǷ
	 * Ƿ
	 */
	public void addProperty(String key, Object value) {
		if(key!=null && value!=null) {
			// get old value, if it equals new value, don't add
			Object oldValue = properties.get(key);
			if(oldValue != null && oldValue.equals(value)) return;
			properties.put(key, value);
			int count = listeners.size();
			// if this property is already exist, trigger property modified event
			//    otherwise trigger property added event
			if(oldValue==null) {
				for(int i=0;i<count;i++)
					((PropertyListener)listeners.get(i)).propertyAdded(new PropertyEvent(this, key, null, value));				
			} else {
				for(int i=0;i<count;i++)
					((PropertyListener)listeners.get(i)).propertyModified(new PropertyEvent(this, key, oldValue, value));
			}
		}
	}

	/* (non-Javadoc)
	 * @see edu.tsinghua.shutter.GeneralItem#getProperty(java.lang.String)
	 */
	public Object getProperty(String key) {
		return properties.get(key);
	}

	/* (non-Javadoc)
	 * @see edu.tsinghua.shutter.GeneralItem#removeProperty(java.lang.String)
	 */
	public void removeProperty(String key) {
		// if this property isn't exist, do nothing
		if(!properties.containsKey(key)) return;
		// get old value
		Object oldValue = properties.remove(key);
		// trigger event
		int count = listeners.size();
		for(int i=0;i<count;i++)
			((PropertyListener)listeners.get(i)).propertyRemoved(new PropertyEvent(this, key, oldValue, null));
	}
	
	/* (non-Javadoc)
	 * @see edu.tsinghua.swt.models.GeneralItem#addPropertyListener(edu.tsinghua.swt.events.PropertyListener)
	 */
	public void addPropertyListener(PropertyListener listener) {
		listeners.add(listener);
	}

	/* (non-Javadoc)
	 * @see edu.tsinghua.swt.models.GeneralItem#removePropertyListener(edu.tsinghua.swt.events.PropertyListener)
	 */
	public void removePropertyListener(PropertyListener listener) {
		listeners.remove(listener);
	}
	
	/* (non-Javadoc)
	 * @see edu.tsinghua.swt.models.INode#hasProperty(java.lang.String)
	 */
	public boolean hasProperty(String key) {
		return properties.containsKey(key);
	}
}
