/*
* Shutter - Friend List Component
*
* Copyright (C) 2004 luma <stubma@163.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
package edu.tsinghua.swt.models;

import java.util.Iterator;

import edu.tsinghua.swt.events.ModelEvent;
import edu.tsinghua.swt.events.ModelListener;

/**
 * Abstract ScrollList model, intended to subclassed by you
 * @author luma@smth
 */
public abstract class AbstractScrollListModel extends AbstractArrayModel implements ScrollListModel {
	/**
	 * constructor
	 */
	public AbstractScrollListModel() {
		super();
	}

	/* (non-Javadoc)
	 * @see edu.tsinghua.shutter.ScrollListModel#getItemCount()
	 */
	public int getItemCount() {
		return nodeArray.size();
	}
	
	/**
	 * trigger model added event
	 * @param eventType
	 */
	protected void fireItemAddedEvent(int itemIndex) {
		Iterator iter = modelListeners.iterator();
		while(iter.hasNext())
			((ModelListener)iter.next()).modelAdded(new ModelEvent(this, itemIndex));
	}
	
	/**
	 * trigger model added event
	 * @param eventType
	 */
	protected void fireItemRemovedEvent(int itemIndex) {
		Iterator iter = modelListeners.iterator();
		while(iter.hasNext())
			((ModelListener)iter.next()).modelRemoved(new ModelEvent(this, itemIndex));
	}
	
	/* (non-Javadoc)
	 * @see edu.tsinghua.swt.models.ScrollListModel#addItem(edu.tsinghua.swt.models.ScrollListItem)
	 */
	public void addItem(INode item) {
		super.add(item);
		fireItemAddedEvent(nodeArray.size() - 1);
	}
	
	/* (non-Javadoc)
	 * @see edu.tsinghua.swt.models.ScrollListModel#insertItem(int, edu.tsinghua.swt.models.ScrollListItem)
	 */
	public boolean insertItem(int index, INode item) {
		boolean succeed = super.insert(index, item);
		if(succeed)
			fireItemAddedEvent(index);
		return succeed;
	}

	/* (non-Javadoc)
	 * @see edu.tsinghua.swt.models.ScrollListModel#removeItem(int)
	 */
	public INode removeItem(int index) {
		// if remove succeed, trigger event
		INode ret = super.remove(index);
		if(ret != null) fireItemRemovedEvent(index);
		return ret;
	}
	
	/* (non-Javadoc)
	 * @see edu.tsinghua.swt.models.ScrollListModel#getItem(int)
	 */
	public INode getItem(int index) {
		return super.get(index);
	}
	
	/* (non-Javadoc)
	 * @see edu.tsinghua.swt.models.ScrollListModel#findItemIndex(edu.tsinghua.swt.models.IMatcher)
	 */
	public int[] findItemIndex(IMatcher matcher) {
		return super.findIndex(matcher);
	}
}
