/*
* LumaQQ - Java QQ Client
*
* Copyright (C) 2004 luma <stubma@163.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
package edu.tsinghua.swt.models;

import java.util.Iterator;
import java.util.LinkedHashMap;

import edu.tsinghua.swt.internal.utils.Utils;

/**
 * <pre>
 * ȱʡֶṩʵ֡ṩ˻湦ܣcacheмǷѾһУ
 * ֱӷءΪ˲ڴcacheеݳһֵʱԶһ
 * </pre>
 * 
 * @author 
 */
public class CachedFieldProvider implements IFieldProvider {
    // Logл
    private LinkedHashMap cache;
    // ָ
    private char[] delimits;
    private char containerChar;
    // Ƿָ
    private boolean includeContainer;
    // ֵ
    private static final int THRESHOLD = 10000;
    
    /**
     * 캯
     */
    public CachedFieldProvider() {
        cache = new LinkedHashMap(500, 0.75f, true);
        delimits = Utils.DELIMITS;
        containerChar = Utils.CONTAINER;
        includeContainer = true;
    }
    
    /**
     * 캯
     * 
     * @param delimits
     * 			ָ
     * @param cc
     * 			
     * @param includeContainer
     * 			Ƿ
     */
    public CachedFieldProvider(char[] delimits, char cc, boolean includeContainer) {
        cache = new LinkedHashMap(500, 0.75f, true);
        this.delimits = delimits;
        this.containerChar = cc;
        this.includeContainer = includeContainer;
    }
    
    /* (non-Javadoc)
     * @see edu.tsinghua.swt.models.IFieldProvider#getFields(java.lang.String)
     */
    public String[] getFields(String row) {
        Object ret = cache.get(row);
        if(ret == null) {
            String[] fields = Utils.splitString(row, delimits, containerChar, includeContainer);
            cache.put(row, fields);
            if(cache.size() > THRESHOLD)
                cleanUp();
            return fields;
        } else
            return (String[])ret;
    }

    /**
     * Cacheһ
     */
    private void cleanUp() {
        int clean = cache.size() >> 1;
        Iterator iter = cache.keySet().iterator();
        while(clean-- > 0) {
            iter.next();
            iter.remove();
        }
    }

    /* (non-Javadoc)
     * @see edu.tsinghua.swt.models.IFieldProvider#getField(java.lang.String, int)
     */
    public String getField(String row, int fieldIndex) {
        String[] fields = getFields(row);
        if(fields != null && fieldIndex < fields.length)
            return fields[fieldIndex];
        else
            return null;
    }

    /* (non-Javadoc)
     * @see edu.tsinghua.swt.models.IFieldProvider#setIncludeContainerChar(boolean)
     */
    public void setIncludeContainerChar(boolean b) {
        this.includeContainer = b;
    }

    /* (non-Javadoc)
     * @see edu.tsinghua.swt.models.IFieldProvider#setDelimits(char[])
     */
    public void setDelimits(char[] delimits) {
        this.delimits = delimits;
    }

    /* (non-Javadoc)
     * @see edu.tsinghua.swt.models.IFieldProvider#setContainerChar(char)
     */
    public void setContainerChar(char cc) {
        this.containerChar = cc;
    }
}
