/*
* Shutter - Friend List Component
*
* Copyright (C) 2004 luma <stubma@163.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
package edu.tsinghua.swt.models;

import org.eclipse.swt.graphics.Image;

import edu.tsinghua.swt.internal.resources.Icons;
import edu.tsinghua.swt.widgets.MySWT;

/**
 * Default ScrollList Model implementation
 * @author luma@smth
 */
public class DefaultScrollListModel extends AbstractScrollListModel {	
	/**
	 * constructor
	 */
	public DefaultScrollListModel() {
		super();
	}

	/**
	 * This method add an item which has default small and large image
	 * @param name item's name
	 */
	public void addItem(String name) {
		addItem(name, Icons.getInstance().getImage(MySWT.DEFAULT_IMAGE));
	}
	
	/**
	 * add an item
	 * @param name item's name
	 * @param image item's small image
	 */
	public void addItem(String name, Image image) {
		super.add(new ScrollListItem(name, image));
	}

	/**
	 * This method inserts a item which has default small and large images
	 * @param index index into which the item is inserted
	 * @param name item's name
	 * @return true if insert is successful, false if the index is out of bound
	 */
	public boolean insertItem(int index, String name) {
		return insertItem(index, name, Icons.getInstance().getImage(MySWT.DEFAULT_IMAGE));
	}
	
	/**
	 * insert an item with specified index
	 * @param index index into which the item is inserted
	 * @param name item's name
	 * @param image item's small image
	 * @return true if insert is successful, false if the index is out of bound
	 */
	public boolean insertItem(int index, String name, Image image) {
		return super.insert(index, new ScrollListItem(name, image));
	}

	/* (non-Javadoc)
	 * @see edu.tsinghua.shutter.ScrollListModel#getItemName(int)
	 */
	public String getItemName(int index) {
		if(index < 0 || index >= nodeArray.size()) return null;
		String name = (String)getItem(index).getProperty("name");
		if(name==null)
			name = "Noname";
		return name;
	}

	/* (non-Javadoc)
	 * @see edu.tsinghua.shutter.ScrollListModel#getItemSmallImage(int)
	 */
	public Image getItemImage(int index) {
		if(index < 0 || index >= nodeArray.size()) return null;
		return (Image)getItem(index).getProperty("image");
	}

	/* (non-Javadoc)
	 * @see edu.tsinghua.swt.models.ScrollListModel#setItemName(int, java.lang.String)
	 */
	public void setItemName(int index, String name) {
		if(index < 0 || index >= nodeArray.size()) return;
		getItem(index).addProperty("name", name);
	}

	/* (non-Javadoc)
	 * @see edu.tsinghua.swt.models.ScrollListModel#setItemSmallImage(int, org.eclipse.swt.graphics.Image)
	 */
	public void setItemImage(int index, Image image) {
		if(index < 0 || index >= nodeArray.size()) return;
		getItem(index).addProperty("image", image);
	}
}
