/*
* Shutter - Friend List Component
*
* Copyright (C) 2004 luma <stubma@163.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
package edu.tsinghua.swt.models;

import org.eclipse.swt.graphics.Image;

import edu.tsinghua.swt.internal.resources.Icons;
import edu.tsinghua.swt.widgets.MySWT;

/**
 * Default shutter data model implementation
 * @author luma@smth
 */
public class DefaultShutterModel extends AbstractShutterModel {
	/**
	 * default constructor
	 */
	public DefaultShutterModel() {
		super();
	}

	/**
	 * This method add a tab which has default image
	 * @param name tab's name
	 */
	public void addTab(String name) {
		addTab(name, Icons.getInstance().getImage(MySWT.DEFAULT_IMAGE));
	}
	
	/**
	 * add a tab
	 * @param name tab's name
	 * @param image tab's image
	 */
	public void addTab(String name, Image image) {
		super.addTab(new ShutterTab(name, image));
	}
	
	/**
	 * This method inserts a tab which has a default image
	 * @param index index into which the tab is inserted
	 * @param name tab's name
	 * @return true if insert is successful, false if the index is out of bound
	 */
	public boolean insertTab(int index, String name) {
		return insertTab(index, name, Icons.getInstance().getImage(MySWT.DEFAULT_IMAGE));
	}
	
	/**
	 * insert a tab with specified index
	 * @param index index into which the item is inserted
	 * @param name tab's name
	 * @param image tab's image
	 * @return true if insert is successful, false if the index is out of bound
	 */
	public boolean insertTab(int index, String name, Image image) {
		return super.insertTab(index, new ShutterTab(name, image));
	}

	/* (non-Javadoc)
	 * @see edu.tsinghua.shutter.ShutterModel#getItemName(int, int)
	 */
	public String getItemName(int tabIndex, int itemIndex) {
		if(tabIndex < 0 || itemIndex < 0 || tabIndex >= nodeArray.size() || itemIndex >= getItemCount(tabIndex))
			return null;
		return ((ScrollListModel)tabModels.get(tabIndex)).getItemName(itemIndex);
	}

	/* (non-Javadoc)
	 * @see edu.tsinghua.shutter.ShutterModel#getItemSmallImage(int, int)
	 */
	public Image getItemImage(int tabIndex, int itemIndex) {
		if(tabIndex < 0 || itemIndex < 0 || tabIndex >= nodeArray.size() || itemIndex >= getItemCount(tabIndex))
			return null;
		return ((ScrollListModel)tabModels.get(tabIndex)).getItemImage(itemIndex);
	}

	/* (non-Javadoc)
	 * @see edu.tsinghua.shutter.ShutterModel#getTabImage(int)
	 */
	public Image getTabImage(int index) {
		if(index < 0 || index >= nodeArray.size()) return null;
		return (Image)getTab(index).getProperty("image");
	}

	/* (non-Javadoc)
	 * @see edu.tsinghua.shutter.ShutterModel#getTabName(int)
	 */
	public String getTabName(int index) {
		if(index < 0 || index >= nodeArray.size()) return null;
		return (String)getTab(index).getProperty("name");
	}
	
	/**
	 * This method add an item which has default small and large images to the tab's bottom
	 * @param tabIndex index of tab to which item is added
	 * @param name item's name
	 */
	public void addItem(int tabIndex, String name) {
		addItem(tabIndex, name, Icons.getInstance().getImage(MySWT.DEFAULT_IMAGE));
	}
	
	/**
	 * add an item to a tab's bottom
	 * @param tabIndex index of tab to which item is added
	 * @param name item's name
	 * @param image item's small image 
	 */
	public void addItem(int tabIndex, String name, Image image) {
		if(tabIndex < 0 || tabIndex >= nodeArray.size()) return;	
		((ScrollListModel)tabModels.get(tabIndex)).addItem(new ScrollListItem(name, image));
		fireItemAddedEvent(tabIndex, getItemCount(tabIndex) - 1);
	}
	
	/**
	 * This method inserts an item which has default small and large images
	 * @param tabIndex index into which the tab is inserted
	 * @param itemIndex index into which the item is inserted
	 * @param name item's name
	 * @return true if insert is successful, false if the index is out of bound
	 */
	public boolean insertItem(int tabIndex, int itemIndex, String name) {
		return insertItem(tabIndex, itemIndex, name, Icons.getInstance().getImage(MySWT.DEFAULT_IMAGE));
	}
	
	/**
	 * insert an item into a tab
	 * @param tabIndex index into which the tab is inserted
	 * @param itemIndex index into which the item is inserted
	 * @param name item's name
	 * @param image item's small image
	 * @return true if insert is successful, false if the index is out of bound
	 */
	public boolean insertItem(int tabIndex, int itemIndex, String name, Image image) {
		if(tabIndex < 0 || tabIndex >= nodeArray.size()) return false;	
		boolean succeed = ((ScrollListModel)tabModels.get(tabIndex)).insertItem(itemIndex, new ScrollListItem(name, image));
		if(succeed)
			fireItemAddedEvent(tabIndex, itemIndex);
		return succeed;
	}

	/* (non-Javadoc)
	 * @see edu.tsinghua.swt.models.ShutterModel#setTabName(int, java.lang.String)
	 */
	public void setTabName(int index, String name) {
		if(index < 0 || index >= nodeArray.size()) return;
		getTab(index).addProperty("name", name);
	}

	/* (non-Javadoc)
	 * @see edu.tsinghua.swt.models.ShutterModel#setTabImage(int, org.eclipse.swt.graphics.Image)
	 */
	public void setTabImage(int index, Image image) {
		if(index < 0 || index >= nodeArray.size()) return;
		getTab(index).addProperty("image", image);
	}
}
