/*
* Shutter - Friend List Component
*
* Copyright (C) 2004 luma <stubma@163.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
package edu.tsinghua.swt.models;

import java.util.List;

import edu.tsinghua.swt.events.*;

/**
 * General model, can meet to most situation
 * It only has basic method, such as get, add and remove, so the node locating is
 * performed by its dedicated implementor.
 * The node is also a general abstraction that a node has several key-value pair.
 * The internal data structure isn't specified in this class, it's up to you 
 * implementation. In the distribution, i implement a array-based model called
 * AbstractArrayModel from which the shutter's model extends,  so if you need
 * a tree-based model, what you will write maybe an AbstractTreeModel 
 * @author luma@smth
 */
public interface IModel {	
	/**
	 * һitemûṩ㷨ľʵ֣ҲʵMatcherӿڣݸ
	 * find
	 * @param matcher IMatcher
	 * @return GeneralNodelistûƥΪnullsizeΪ0
	 */
	public List find(IMatcher matcher);
	
	/**
	 * add an item
	 * @param item
	 */
	public void add(INode node);
	
	/**
	 * remove a node according to its property
	 * @param node
	 * @return if remove success, return true; otherwise return false
	 */
	public boolean remove(INode node);

	/**
	 * add a model listener
	 * @param listener
	 */
	public void addModelListener(ModelListener listener);
	
	/**
	 * remove a model listener
	 * @param listener
	 */
	public void removeModelListener(ModelListener listener);
	
	/**
	 * add a item property listener
	 * @param listener
	 */
	public void addPropertyListener(PropertyListener listener);
	
	/**
	 * remove a item property listener
	 * @param listener
	 */
	public void removePropertyListener(PropertyListener listener);
}
