/*
* Shutter - Friend List Component
*
* Copyright (C) 2004 luma <stubma@163.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
package edu.tsinghua.swt.models;

import org.eclipse.swt.graphics.Image;

/**
 * ScrollList model interface, add its own method
 * @author luma@smth
 */
public interface ScrollListModel extends IModel {
	/**
	 * get item's name
	 * @param index item index, 0 is first
	 * @return
	 */
	public String getItemName(int index);
	
	/**
	 * set item's name
	 * @param index
	 * @param name
	 */
	public void setItemName(int index, String name);
	
	/**
	 * get item's small image
	 * @param index item index, 0 is first
	 * @return
	 */
	public Image getItemImage(int index);
	
	/**
	 * set item's small image
	 * @param index
	 * @param image
	 */
	public void setItemImage(int index, Image image);
	
	/**
	 * get item object
	 * @param index item's index
	 * @return item object
	 */
	public INode getItem(int index);
	
	/**
	 * add an item to the last
	 * @param item item object
	 */
	public void addItem(INode item);
	
	/**
	 * insert an item with specified index
	 * @param index insert index
	 * @param item item object
	 * @return true if insert is successful, false if the index is out of bound
	 */
	public boolean insertItem(int index, INode item);
	
	/**
	 * remove an item which is specified by index
	 * @param index item's index 
	 * @return return removed INode if remove success, otherwise return null
	 */
	public INode removeItem(int index);
	
	/**
	 * return item count
	 * @return
	 */
	public int getItemCount();
	
	/**
	 * get index of a node
	 * @param node node object
	 * @return node index
	 */
	public int indexOf(Object node);
	
	/**
	 * еitemǵindex
	 * @param matcher IMatcher
	 * @return indexint
	 */
	public int[] findItemIndex(IMatcher matcher);
}
