/*
* Shutter - Friend List Component
*
* Copyright (C) 2004 luma <stubma@163.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
package edu.tsinghua.swt.models;

import java.util.List;

import org.eclipse.swt.graphics.Image;

/**
 * Shutter model interface
 * @author luma@smth
 */
public interface ShutterModel extends IModel {
	/**
	 * get shutter tab's name
	 * @param index tab index, from top to bottom, 0 is first
	 * @return
	 */
	public String getTabName(int index);
	
	/**
	 * set shutter tab's name
	 * @param index
	 * @param name
	 */
	public void setTabName(int index, String name);
	
	/**
	 * get shutter tab's image
	 * @param index tab index, from top to bottom, 0 is first
	 * @return
	 */
	public Image getTabImage(int index);
	
	/**
	 * set shutter tab's image
	 * @param index
	 * @param image
	 */
	public void setTabImage(int index, Image image);
	
	/**
	 * get item's name in that tab which is specified by tabIndex
	 * @param tabIndex tab index, from top to bottom, 0 is first
	 * @param itemIndex item index in this tab, from top to bottom, 0 is first
	 * @return
	 */
	public String getItemName(int tabIndex, int itemIndex);
	
	/**
	 * get item's small image in that tab which is specified by tabIndex
	 * @param tabIndex tab index, from top to bottom, 0 is first
	 * @param itemIndex item index in this tab, from top to bottom, 0 is first
	 * @return
	 */
	public Image getItemImage(int tabIndex, int itemIndex);
	
	/**
	 * get tab object which is specified by index
	 * @param index tab's index
	 * @return tab object
	 */
	public INode getTab(int index);
	
	/**
	 * add a tab to the last
	 * @param tab tab object
	 */
	public void addTab(INode tab);
	
	/**
	 * insert a tab with specified index
	 * @param index insert index
	 * @param tab tab object
	 * @return true if insert is successful, false if the index is out of bound
	 */
	public boolean insertTab(int index, INode tab);
	
	/**
	 * remove a tab which is specified by index
	 * @param index tab's index
	 * @return return removed tab is remove is success, otherwise return null
	 */
	public INode removeTab(int index);
	
	/**
	 * add an item to the bottom of a tab which is specified by tabIndex
	 * @param tabIndex tab index
	 * @param item item object
	 */
	public void addItem(int tabIndex, INode item);
	
	/**
	 * insert an item to the specified location of a tab which is specified by tabIndex
	 * @param tabIndex tab index
	 * @param itemIndex item index
	 * @param item item object
	 * @return true if insert is successful, false if the index is out of bound
	 */
	public boolean insertItem(int tabIndex, int itemIndex, INode item);
	
	/**
	 * remove an item from a tab which is specified by tabIndex
	 * @param tabIndex tab index
	 * @param itemIndex item index
	 * @return return removed item if remove success, otherwise return null
	 */
	public INode removeItem(int tabIndex, int itemIndex);
	
	/**
	 * get an item object
	 * @param tabIndex tab index, 0 is first
	 * @param itemIndex item index, 0 is first
	 */
	public INode getItem(int tabIndex, int itemIndex);
	
	/**
	 * get a ScrollListModel corresponding to a tab
	 * @param tabIndex tab index
	 * @return ScrollListModel object belongs to this tab
	 */
	public ScrollListModel getTabModel(int tabIndex);
	
	/**
	 * set a tab's model
	 * @param tabIndex tab index
	 * @param model ScrollListModel object
	 */
	public void setTabModel(int tabIndex, ScrollListModel model);
	
	/**
	 * get item count in a tab which is specified by tabIndex
	 * @param tabIndex tab index, from top to bottom, 0 is first
	 * @return how many items in a tab
	 */
	public int getItemCount(int tabIndex);
	
	/**
	 * @return the number of all items
	 */
	public int getAllItemCount();
	
	/**
	 * get tab count
	 * @return tab count
	 */
	public int getTabCount();
	
	/**
	 * itemûһMatcher˷
	 * @param matcher IMatcher
	 * @return ItemGeneralNode List
	 */
	public List findItem(IMatcher matcher);
	
	/**
	 * еTabǵindex
	 * @param matcher IMatcher
	 * @return indexint
	 */
	public int[] findTabIndex(IMatcher matcher);
	
	/**
	 * һtabitemǵindex
	 * @param tabIndex Ҫtab
	 * @param matcher IMatcher
	 * @return indexint
	 */
	public int[] findItemIndex(int tabIndex, IMatcher matcher);
	
	/**
	 * õһTabindex
	 * @param tab tab
	 * @return index
	 */
	public int indexOf(Object tab);
	
	/**
	 * tabIndexָtabеõһitemindexûУ-1
	 * @param item
	 * @return
	 */
	public int indexOfItem(int tabIndex, Object item);
}
