/*
* Shutter - Friend List Component
*
* Copyright (C) 2004 luma <stubma@163.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
package edu.tsinghua.swt.widgets;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Iterator;

import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DragSourceListener;
import org.eclipse.swt.dnd.DropTargetListener;
import org.eclipse.swt.events.ControlAdapter;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.ControlListener;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.MouseMoveListener;
import org.eclipse.swt.events.MouseTrackListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;

import edu.tsinghua.swt.events.CoolButtonEvent;
import edu.tsinghua.swt.events.CoolButtonListener;
import edu.tsinghua.swt.events.ModelEvent;
import edu.tsinghua.swt.events.ModelListener;
import edu.tsinghua.swt.events.PropertyEvent;
import edu.tsinghua.swt.events.PropertyListener;
import edu.tsinghua.swt.internal.layout.DefaultNodeComparator;
import edu.tsinghua.swt.internal.resources.Icons;
import edu.tsinghua.swt.internal.utils.FirstMatcher;
import edu.tsinghua.swt.layout.SortData;
import edu.tsinghua.swt.layout.SortLayout;
import edu.tsinghua.swt.models.IExtraWork;
import edu.tsinghua.swt.models.IMatcher;
import edu.tsinghua.swt.models.INode;
import edu.tsinghua.swt.models.ScrollListModel;

/**
 * @author luma@smth
 *
 * ScrollList is scrollable composite, but no ugly scrollbar
 * it's a QQ-like style scroll list
 */
public class ScrollList extends Composite implements DisposeListener, ControlListener, ModelListener, PropertyListener, CoolButtonListener {
	// private class
	//    when use click on scroll list, it will get keyboard input
	private class ScrollListDefaultMouseListener extends MouseAdapter {		
		/* (non-Javadoc)
		 * @see org.eclipse.swt.events.MouseListener#mouseUp(org.eclipse.swt.events.MouseEvent)
		 */
		public void mouseUp(MouseEvent e) {
			forceFocus();
		}
	}
	
	// private class
	//    when content is moved, invoke this listener
	//    this listener will keep content in its corrent location
	private class ContentControlListener extends ControlAdapter {
		public void controlMoved(ControlEvent e) {
			if(contentAlignment == SWT.LEFT)
				content.setLocation(0, contentY);
			else if(contentAlignment == SWT.RIGHT)
				content.setLocation(getParent().getSize().x - content.getSize().x, contentY);
			else if(contentAlignment == SWT.CENTER)
				content.setLocation((getParent().getSize().x - content.getSize().x) / 2, contentY);
		}
	}
	
	// private class
	//    it's content key listener, then we can use keyboard scroll now
	private class ScrollKeyListener extends KeyAdapter {
		// old scroll step size
		private int oldStepSize;
		
		/* (non-Javadoc)
		 * @see java.awt.event.KeyListener#keyPressed(java.awt.event.KeyEvent)
		 */
		public void keyPressed(KeyEvent e) {
			// check keyCode, we process Up key, Down key, PageUp key and PageDown key
			//    if keyCode is PageUp or PageDown, we store old scroll step size, set
			//    a new step size = 100, so PageUp and PageDown will scroll fast
			if(stepSize != 100)
				oldStepSize = stepSize;
			switch(e.keyCode) {
				case SWT.ARROW_UP:
					up = true;
					break;
				case SWT.ARROW_DOWN:
					up = false;
					break;
				case SWT.PAGE_UP:
					up = true;
					stepSize = 100;
					break;
				case SWT.PAGE_DOWN:
					up = false;
					stepSize = 100;
					break;
				case SWT.HOME:
				    contentY = 0;
					content.setLocation(0, contentY);
					adjustButtonVisibility();
					return;
				case SWT.END:
				    contentY = clientArea.height - contentRect.height;
					if(contentY > 0)
					    contentY = 0;
					content.setLocation(0, contentY);
					adjustButtonVisibility();
					return;
				default:
				    if(!Character.isISOControl(e.character))
				        revealItem(e.character);
				    return;		    
			}
			// begin scroll
		    setScrolling(true);
		    display.timerExec(0, scrollRunnable);
		}

		/* (non-Javadoc)
		 * @see java.awt.event.KeyListener#keyReleased(java.awt.event.KeyEvent)
		 */
		public void keyReleased(KeyEvent e) {
			// if it's PageUp or PageDown, restore the old step size
			if(e.keyCode==SWT.PAGE_DOWN || e.keyCode==SWT.PAGE_UP)
				stepSize = oldStepSize;
			// stop scroll
			setScrolling(false);
		}
	}
	
	// ִйrunnable
	private Runnable scrollRunnable = new Runnable() {
		public void run() {
		    // contentYֵ
			if(up) contentY += stepSize;
			else contentY -= stepSize;
			// check contentY value
			//    if contentY > 0, then there is blank above content, this isn't permitted.
			//    if list is now down-scrolling and the down button is invisible, wait, how
			//    could this situation possible? because keyboard! so under this situation,
			//    we cancel the contentY's change
			//    if list is now down-scrolling, and there is blank below the content, and 
			//    the content height is larger than client area, and down button is visible,
			//    then the blank below the content isn't permitted. But if any of them is 
			//    false, we do nothing.
			if(contentY > 0)
				contentY = 0;
			if(!up && btnDown.getVisible()==false)
				contentY += stepSize;
			if(!up && contentY + contentRect.height < clientArea.height && contentRect.height > clientArea.height && btnDown.getVisible())
				contentY = clientArea.height - contentRect.height;
			
			// scroll
			content.setLocation(0, contentY);
			
			// set up and down button visibility
			adjustButtonVisibility();
			
			// adjust scroll status
			if(contentY >= 0)
				setScrolling(false);
			if(contentY + contentRect.height <= clientArea.height && !up)
					setScrolling(false);
			nextScroll();
		}						
	};
	
	// private class
	// deal with up and down button's mouse events
	private class ScrollButtonMouseListener extends MouseAdapter {		
		/* (non-Javadoc)
		 * @see org.eclipse.swt.events.MouseListener#mouseDown(org.eclipse.swt.events.MouseEvent)
		 */
		public void mouseDown(MouseEvent e) {
			// check what button is pressed
			String button = (String)((Button)e.getSource()).getData();
			if(button.equals("Up")) up = true;
			else up = false;
			// begin scroll
		    setScrolling(true);
		    display.timerExec(0, scrollRunnable);
		}

		/* (non-Javadoc)
		 * @see org.eclipse.swt.events.MouseListener#mouseUp(org.eclipse.swt.events.MouseEvent)
		 */
		public void mouseUp(MouseEvent e) {
			setScrolling(false);
			forceFocus();
		}
	}
	
	/**
	 * Ҫ趨һʱٴι
	 */
	private synchronized void nextScroll() {
	    if(scrolling)
	        display.timerExec(scrollSpeed, scrollRunnable);
	}
	
	/**
	 * ¹ťɼ״̬
	 */
	private void adjustButtonVisibility() {
		// set up and down button visibility
		if(contentY < 0)
			btnUp.setVisible(true);
		else {
			// if we at the top, we set up button invisible
			btnUp.setVisible(false);
		}
		if(contentY + contentRect.height > clientArea.height)
			btnDown.setVisible(true);
		else {
			// if we at the end, we set down button invisible
			btnDown.setVisible(false);
		}
	}
	
	/**
	 * ñ־Ƿ
	 * @param b
	 */
	private synchronized void setScrolling(boolean b) {
	    scrolling = b;
	}
	
	// content composite
	private Composite content;
	// current vertical position
	private int contentY;
	// interval between two scrolls
	private int scrollSpeed;
	// pixels per scroll
	private int stepSize;
	// flag of which button is pressed
	private boolean up;
	// Ƿڹ
	private volatile boolean scrolling;
	// current display
	private Display display;
	// up button and down button
	private Button btnUp, btnDown;
	// content bound and scrolllist client area
	private Rectangle contentRect, clientArea;
	// scroll list model
	private ScrollListModel model;
	// item's control
	private ArrayList itemControls;
	// item's mouse track listeners
	private ArrayList mouseTrackListeners;
	// item's mouse move listeners
	private ArrayList mouseMoveListeners;
	// item's mouse listeners
	private ArrayList mouseListeners;
	// item's drag source listeners
	private ArrayList dragSourceListeners;
	// item's drop target listeners
	private ArrayList dropTargetListeners;
	// clients who listen model and property event
	private ArrayList modelListeners;
	private ArrayList propertyListeners;
	// Ƶitemʱʾɫ
	private Color mouseTrackColor;
	// Extra Work
	private IExtraWork extraWork;
	// item
	private IMatcher filter;
	// current image mode
	private boolean showSmall;
	// ɫ
	private Color contentBackground;
	// whether show text or not
	private boolean showText;
	// text orientation
	private int textOrientation;
	// content alignment
	private int contentAlignment;
	// ĸƥ
	private static FirstMatcher firstMatcher = new FirstMatcher();
	
	/**
	 * constructor
	 * @param parent
	 * @param style
	 */
	public ScrollList(Composite parent, int style) {
		super(parent, SWT.NONE);
		
		// init layout
		initLayout();		
		
		// initialize some variable
		itemControls = new ArrayList();
		mouseTrackListeners = new ArrayList();
		mouseMoveListeners = new ArrayList();
		mouseListeners = new ArrayList();
		modelListeners = new ArrayList();
		propertyListeners = new ArrayList();
		dragSourceListeners = new ArrayList();
		dropTargetListeners = new ArrayList();
		
		// get current display
		display = getDisplay();
		// set default scroll speed
		scrollSpeed = 50;
		stepSize = 30;
		// set show with small image
		showSmall = true;
		showText = true;
		textOrientation = SWT.RIGHT;
		contentBackground = null;
		contentAlignment = SWT.LEFT;
		// initialize scroll flag
		scrolling = false;
		
		// add this as following listeners
		addDisposeListener(this);
		addControlListener(this);
		
		// initialize icons
		Icons.getInstance().loadResources();
	}
	
	/**
	 * init scroll list layout and add two button
	 */
	public void initLayout() {
		// set layout
		FormLayout layout = new FormLayout();
		layout.marginHeight = 0;
		layout.marginWidth = 0;
		this.setLayout(layout);		
		
		// add up and down button and set their layout data
		btnUp = new Button(this, SWT.ARROW | SWT.UP);
		FormData fd = new FormData();
		fd.top = new FormAttachment(0, 5);
		fd.right = new FormAttachment(100, -5);
		btnUp.setLayoutData(fd);
		btnUp.setData("Up");
		btnDown = new Button(this, SWT.ARROW | SWT.DOWN);
		fd = new FormData();
		fd.bottom = new FormAttachment(100, -5);
		fd.right = new FormAttachment(100, -5);
		btnDown.setLayoutData(fd);
		btnDown.setData("Down");
		
		// move button to top, so the button will not covered by content
		btnUp.moveAbove(content);
		btnDown.moveAbove(content);
		
		// add mouse listener
		ScrollButtonMouseListener listener = new ScrollButtonMouseListener();
		btnUp.addMouseListener(listener);
		btnDown.addMouseListener(listener);
		
		// set button invisible, because no item in ScrollList
		btnUp.setVisible(false);
		btnDown.setVisible(false);
	}
	
	/**
	 * ϷԴ
	 * @param listener
	 */
	public void addItemDragSourceListener(DragSourceListener listener) {
		dragSourceListeners.add(listener);
		int size = itemControls.size();
		for(int i = 0; i < size; i++)
			((CoolButton)itemControls.get(i)).addDragSourceListener(listener);
	}
	
	/**
	 * ɾϷԴ
	 * @param listener
	 */
	public void removeItemDragSourceListener(DragSourceListener listener) {
		dragSourceListeners.remove(listener);
		int size = itemControls.size();
		for(int i = 0; i < size; i++)
			((CoolButton)itemControls.get(i)).removeDragSourceListener(listener);
	}
	
	/**
	 * ϷĿļ
	 * @param listener
	 */
	public void addItemDropTargetListener(DropTargetListener listener) {
		dropTargetListeners.add(listener);
		int size = itemControls.size();
		for(int i = 0; i < size; i++)
			((CoolButton)itemControls.get(i)).addDropTargetListener(listener);
	}
	
	/**
	 * ɾϷĿļ
	 * @param listener
	 */
	public void removeItemDropTargetListener(DropTargetListener listener) {
		dropTargetListeners.remove(listener);
		int size = itemControls.size();
		for(int i = 0; i < size; i++)
			((CoolButton)itemControls.get(i)).removeDropTargetListener(listener);
	}
	
	/**
	 * add a general model listener
	 * @param listener
	 */
	public void addModelListener(ModelListener listener) {
		modelListeners.add(listener);
	}
	
	/**
	 * remove a general model listener
	 * @param listener
	 */
	public void removeModelListener(ModelListener listener) {
		modelListeners.remove(listener);
	}
	
	/**
	 * add a property listener
	 * @param listener
	 */
	public void addPropertyListener(PropertyListener listener) {
		propertyListeners.add(listener);
	}
	
	/**
	 * remove a property listener
	 * @param listener
	 */
	public void removePropertyListener(PropertyListener listener) {
		propertyListeners.remove(listener);
	}
	
	/**
	 * add key listener to content, also add to this
	 * because we want user think they are same
	 * @param listener
	 */
	public void addKeyListener(KeyListener listener) {
		if(listener != null && content != null) {
			content.addKeyListener(listener);
			super.addKeyListener(listener);			
		}
	}
	
	/**
	 * remove content's key listener, also remove from this
	 * because we want user think they are same
	 * @param listener
	 */
	public void removeKeyListener(KeyListener listener) {
		if(listener != null && content != null) {
			content.removeKeyListener(listener);
			super.removeKeyListener(listener);
		}		
	}
	
	/**
	 * add mouse listener to content, also add to this
	 * because we want user think they are same
	 * @param listener
	 */
	public void addMouseListener(MouseListener listener) {
		if(listener != null && content != null) {
			content.addMouseListener(listener);
			super.addMouseListener(listener);
		}
	}
	
	/**
	 * remove content's mouse listener, also remove from this
	 * because we want user think they are same
	 * @param listener
	 */
	public void removeMouseListener(MouseListener listener) {
		if(listener != null && content != null) {
			content.removeMouseListener(listener);
			super.removeMouseListener(listener);
		}
	}	

	/**
	 * add content's mouse move listener, also add to this
	 * because we want user think they are same
	 * @param listener
	 */
	public void addMouseMoveListener(MouseMoveListener listener) {
		if(listener != null && content != null) {
			content.addMouseMoveListener(listener);
			super.addMouseMoveListener(listener);
		}
	}

	/**
	 * remove content's mouse move listener, also remove from this
	 * because we want user think they are same
	 * @param listener
	 */
	public void removeMouseMoveListener(MouseMoveListener listener) {
		if(listener != null && content != null) {
			content.removeMouseMoveListener(listener);
			super.removeMouseMoveListener(listener);
		}
	}
	
	/**
	 * add content's mouse track listener, also add to this
	 * because we want user think they are same
	 * @param listener
	 */
	public void addMouseTrackListener(MouseTrackListener listener) {
		if(listener != null && content != null) {
			content.addMouseTrackListener(listener);
			super.addMouseTrackListener(listener);
		}
	}

	/**
	 * remove content's mouse track listener, also remove from this
	 * because we want user think they are same
	 * @param listener
	 */
	public void removeMouseTrackListener(MouseTrackListener listener) {
		if(listener != null && content != null) {
			content.removeMouseTrackListener(listener);
			super.removeMouseTrackListener(listener);
		}
	}
	
	/**
	 * add items' mouse track listener which will be invoked when mouse enter or exit an item
	 * @param listener
	 */
	public void addItemMouseTrackListener(MouseTrackListener listener) {
		if(listener != null) {
			Iterator iter = itemControls.iterator();
			while(iter.hasNext())
				((CoolButton)iter.next()).addMouseTrackListener(listener);
			mouseTrackListeners.add(listener);
		}
	}
	
	/**
	 * remove items' mouse track listener
	 * @param listener
	 */
	public void removeItemMouseTrackListener(MouseTrackListener listener) {
		if(listener != null) {
			Iterator iter = itemControls.iterator();
			while(iter.hasNext())
				((CoolButton)iter.next()).removeMouseTrackListener(listener);
			mouseTrackListeners.remove(listener);
		}
	}
	
	/**
	 * add items' mouse move listener which will be invoked when mouse pass an item
	 * @param listener
	 */
	public void addItemMouseMoveListener(MouseMoveListener listener) {
		if(listener != null) {
			Iterator iter = itemControls.iterator();
			while(iter.hasNext())
				((CoolButton)iter.next()).addMouseMoveListener(listener);
			mouseMoveListeners.add(listener);
		}
	}
	
	/**
	 * remove items' mouse move listener
	 * @param listener
	 */
	public void removeItemMouseMoveListener(MouseMoveListener listener) {
		if(listener != null) {
			Iterator iter = itemControls.iterator();
			while(iter.hasNext())
				((CoolButton)iter.next()).removeMouseMoveListener(listener);
			mouseMoveListeners.remove(listener);
		}
	}
	
	/**
	 * add items' mouse listener which will be invoked when mouse click on an item
	 * @param listener
	 */
	public void addItemMouseListener(MouseListener listener) {
		if(listener != null) {
			Iterator iter = itemControls.iterator();
			while(iter.hasNext())
				((CoolButton)iter.next()).addMouseListener(listener);
			mouseListeners.add(listener);
		}
	}
	
	/**
	 * remove items' mouse listener
	 * @param listener
	 */
	public void removeItemMouseListener(MouseListener listener) {
		if(listener != null) {
			Iterator iter = itemControls.iterator();
			while(iter.hasNext())
				((CoolButton)iter.next()).removeMouseListener(listener);
			mouseListeners.remove(listener);
		}
	}
	
	/**
	 * set model
	 * @param model scroll list model
	 */
	public void setModel(ScrollListModel model) {
		// if scroll list has a mode, release it, and remove this from model listener
		if(this.model != null) {
			this.model.removeModelListener(this);
			Iterator iter = itemControls.iterator();
			while(iter.hasNext())
				((CoolButton)iter.next()).dispose();
			itemControls.clear();
		}
		// set scroll list model to this new one
		this.model = model;
		// if new model isn't null, do refresh
		if(this.model != null) {
			// add me as a model listener
			this.model.addModelListener(this);
			// add me as a property listener
			this.model.addPropertyListener(this);
			// initialize items
			initItems();			
		}
		// relayout
		relayout();
	}
	
	/**
	 * get scroll list model
	 * @return a ScrollListModel object associated with scroll list
	 */
	public ScrollListModel getModel() {
		return model;
	}
	
	/**
	 * remove all item
	 */
	public void removeAllItem() {
		int count = itemControls.size();
		for(int i=0;i<count;i++)
			model.removeItem(i);
	}
	
	/**
	 * get item control according to index
	 * @param index item index
	 * @return
	 */
	public CoolButton getItemControl(int index) {
		if(index < 0 || index >= itemControls.size()) return null;
		return (CoolButton)itemControls.get(index);
	}
	
	/**
	 * غnodeӦcontrol
	 * @param node INode
	 * @return nodeӦcontrol
	 */
	public CoolButton getItemControl(INode node) {
		return getItemControl(model.indexOf(node));
	}
	
	/**
	 * return item index
	 * @param item
	 * @return the control's index, the returned index specify control's position in
	 * 		internal array index, so even the items is sorted, the index is also valid
	 * 		and can be passed to getItemControl to get	control
	 */
	public int indexOf(Control item) {
		return itemControls.indexOf(item);
	}
	
	// create items' controls according to model
	private void createControlsFromModel() {
		int count = model.getItemCount();
		for(int i=0;i<count;i++) {
			// new a CoolButton as an item
			CoolButton item = new CoolButton(content, MySWT.FLAT | MySWT.IMAGE_HOVER | SWT.RIGHT, model.getItemName(i), model.getItemImage(i));
			// initialize item
			initItem(item, i);
			// add to array
			itemControls.add(item);
			// жǷӦù˵item
			if(filter != null)
				if(filter.match(model.getItem(i)))
					hideItem(i, true);
		}
		
		// set controls' background
		setControlsBackground(this.getBackground());
	}
	
	// itemʱʼ
	private void initItem(CoolButton item, int index) {
		INode node = model.getItem(index);
		// set layout data
		SortData sd = new SortData(SortData.FILL_HORIZONTAL, index, node);
		item.setLayoutData(sd);
		// add model as data
		item.setData(node);
		// set mouse track color
		if(mouseTrackColor != null) item.setMouseTrackColor(mouseTrackColor);
		// set colorimage modeetc
		if(contentBackground != null) item.setBackground(contentBackground);
		item.setShowSmallImage(showSmall);
		item.setShowText(showText);
		item.setTextOrientation(textOrientation);
		// add this as shutter label event listener
		item.addShutterLabelListener(this);
		// do extra work
		if(extraWork != null)
			extraWork.doExtraWork(item, node);
	}
	
	/**
	 * set image show mode
	 * @param small if show in small, true
	 */
	public void setShowSmallImage(boolean small) {
		showSmall = small;
		int count = model.getItemCount();
		for(int i=0;i<count;i++)
			((CoolButton)itemControls.get(i)).setShowSmallImage(showSmall);
		
		// re-layout, if content is larger than client area, show down button
		relayout();
	}
	
	/**
	 * set items' text orientation
	 * @param orientation
	 */
	public void setTextOrientation(int orientation) {
		textOrientation = orientation;
		int count = itemControls.size();
		for(int i=0;i<count;i++)
			((CoolButton)itemControls.get(i)).setTextOrientation(orientation);
		relayout();
	}
	
	/**
	 * set item's tooltip text
	 * @param text tooltip text
	 * @param index The index of item whose text tooltip should be set. 
	 * 		  If it's MySWT.ALL, set all
	 */
	public void setToolTipText(String text, int index) {
	    if(index == MySWT.ALL) {
	        Iterator iter = itemControls.iterator();
	        while(iter.hasNext())
	            ((CoolButton)iter.next()).setToolTipText(text);
	    } else {
			if(index < 0 || index >=itemControls.size()) return;
			((CoolButton)itemControls.get(index)).setToolTipText(text);	        
	    }
	}
	
	/**
	 * set show item text or not
	 * @param showText
	 */
	public void setShowText(boolean showText) {
		this.showText = showText;
		int count = itemControls.size();
		for(int i=0;i<count;i++)
			((CoolButton)itemControls.get(i)).setShowText(showText);
		relayout();
	}
	
	/**
	 * set content column number
	 * @param cols
	 */
	public void setContentColumns(int cols) {
		SortLayout layout = new SortLayout();
		layout.horizontalSpacing = layout.verticalSpacing = 1;
		layout.makeColumnsEqualWidth = true;
		layout.marginHeight = layout.marginWidth = 1;
		layout.numColumns = cols;
		layout.keyComparator = new DefaultNodeComparator();
		content.setLayout(layout);

		// relayout
		relayout();
	}
	
	/**
	 * set content and initialize location to (0, 0)
	 * @param content
	 * @return old content
	 */
	public Composite setContent(Composite content) {
		// store old content
		Composite oldContent = this.content;
		// set content
		this.content = content;
		content.setBackground(this.getBackground());
		content.setLocation(0, 0);
		// set content layout data
		FormData fd = new FormData();
		fd.left = new FormAttachment(0, 0);
		content.setLayoutData(fd);
		// set content layout
		setContentColumns(1);
		contentY = 0;
		// add content control listener
		content.addControlListener(new ContentControlListener());
		// add following listeners to this and content
		addMouseListener(new ScrollListDefaultMouseListener());
		addKeyListener(new ScrollKeyListener());
		
		// init items
		initItems();
		// return old content
		return oldContent;
	}
	
	// initialize items, including create them, add listener to them
	private void initItems() {
		// create controls
		if(content != null && model != null) {
			// create all controls
			createControlsFromModel();
			
			// add all listenens to items
			addAllListenersToAllItems();		
		}
	}

	/**
	 * ֮ǰӵм
	 */
	public void clearListeners() {
		mouseListeners.clear();
		mouseMoveListeners.clear();
		mouseTrackListeners.clear();
		dragSourceListeners.clear();
		dropTargetListeners.clear();
	}
	
	/**
	 * get content 
	 * @return get content composite, not to use it directly by user 
	 */
	public Composite getContent() {
		return content;
	}
	
	/**
	 * scrolllistitemItemmodelҲGeneralNode
	 * ͽû԰GeneralNodeתԶͽ
	 * ݵҲûԶ
	 * @param sorter 
	 */
	public void setSorter(Comparator sorter) {
		((SortLayout)content.getLayout()).keyComparator = sorter;
	}
	
	/**
	 * set content background, and update controls' backgroud in order to conform to content
	 * @param color
	 */
	public void setContentBackground(Color color) {
		if(content != null) {
			this.contentBackground = color;
			content.setBackground(color);
			this.setBackground(color);
			setControlsBackground(color);			
		}
	}
	
	// set items' background
	private void setControlsBackground(Color color) {
		Iterator iter = itemControls.iterator();
		while(iter.hasNext()) {
			((CoolButton)iter.next()).setBackground(color);
		}
	}

	/**
	 * set scroll speed, i.e., when button is pressed, the interval between two scrolls
	 * the more smaller the value is, the more faster it scrolls
	 * @param speed
	 */
	public void setScrollSpeed(int speed) {
		this.scrollSpeed = speed;
	}
	
	/**
	 * set scroll step size - the number of pixel per scroll
	 * @param stepSize
	 */
	public void setScrollStepSize(int stepSize) {
		this.stepSize = stepSize;
	}
	
	/**
	 * start blink an image on an item which is specified by itemIndex
	 * @param blinkImage image to be blinked
	 * @param itemIndex item index
	 */
	public void startBlinkImage(Image blinkImage, int itemIndex) {
		if(itemIndex < 0 || itemIndex >=itemControls.size()) return;
		((CoolButton)itemControls.get(itemIndex)).startBlinkImage(blinkImage);
	}
	
	/**
	 * stop blinking image
	 * @param itemIndex item index
	 */
	public void stopBlinkImage(int itemIndex) {
		if(itemIndex < 0 || itemIndex >=itemControls.size()) return;
		((CoolButton)itemControls.get(itemIndex)).stopBlinkImage();		
	}
	
	/**
	 * start leap an image on an item which is specified by itemIndex
	 * @param leapImage image to be leaped
	 * @param itemIndex item index
	 */
	public void startLeapImage(Image leapImage, int itemIndex) {
		if(itemIndex < 0 || itemIndex >=itemControls.size()) return;
		((CoolButton)itemControls.get(itemIndex)).startBounceImage(leapImage);
	}
	
	/**
	 * stop leaping image
	 * @param itemIndex item index
	 */
	public void stopLeapImage(int itemIndex) {
		if(itemIndex < 0 || itemIndex >=itemControls.size()) return;
		((CoolButton)itemControls.get(itemIndex)).stopBounceImage();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.DisposeListener#widgetDisposed(org.eclipse.swt.events.DisposeEvent)
	 */
	public void widgetDisposed(DisposeEvent e) {
		// free resource
		Icons.getInstance().freeResources();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.ControlListener#controlMoved(org.eclipse.swt.events.ControlEvent)
	 */
	public void controlMoved(ControlEvent e) {
	}

	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.ControlListener#controlResized(org.eclipse.swt.events.ControlEvent)
	 */
	public void controlResized(ControlEvent e) {
		relayout();
	}
	
	// add all listeners to all items
	private void addAllListenersToAllItems() {
		Iterator iter = itemControls.iterator();
		while(iter.hasNext())
			addAllListenersToAnItem((CoolButton)iter.next());
	}
	
	// add all listeners to an item
	private void addAllListenersToAnItem(CoolButton item) {
		// add mouse track listener
		Iterator iter = mouseTrackListeners.iterator();
		while(iter.hasNext())
			item.addMouseTrackListener((MouseTrackListener)iter.next());
		// add mouse move listener
		iter = mouseMoveListeners.iterator();
		while(iter.hasNext())
			item.addMouseMoveListener((MouseMoveListener)iter.next());
		// add mouse listener
		iter = mouseListeners.iterator();
		while(iter.hasNext())
			item.addMouseListener((MouseListener)iter.next());
		// add drag source listener
		iter = dragSourceListeners.iterator();
		while(iter.hasNext())
			item.addDragSourceListener((DragSourceListener)iter.next());
		// add drop target listener
		iter = dropTargetListeners.iterator();
		while(iter.hasNext())
			item.addDropTargetListener((DropTargetListener)iter.next());
	}
	
	// re-layout, if content is larger than client area, show down button
	private void relayout() {
		// relayout
		this.layout();
		// get content bound, if no content, get all zero
		if(content==null)
			contentRect = new Rectangle(0, 0, 0, 0);
		else
			contentRect = content.getBounds();
		// get client area bound
		clientArea = this.getClientArea();
		// if no item, don't show any button
		// if has, check bound to decide
		if(itemControls.size() > 0) {
			// if content bottom exceed client area bottom, show down button
			if(contentY + contentRect.height > clientArea.height)
				btnDown.setVisible(true);
			else
				btnDown.setVisible(false);			
		} else {
			btnDown.setVisible(false);
			btnUp.setVisible(false);
		}
	}
	
	/* (non-Javadoc)
	 * @see edu.tsinghua.swt.events.ModelListener#modelAdded(edu.tsinghua.swt.events.ModelEvent)
	 */
	public void modelAdded(ModelEvent e) {
		ScrollListModel m = (ScrollListModel)e.getSource();
		// if a item added, create item's counterpart control
		CoolButton item = new CoolButton(content, MySWT.IMAGE_HOVER | MySWT.FLAT | SWT.RIGHT, m.getItemName(e.itemIndex), m.getItemImage(e.itemIndex));
		// initialize item
		initItem(item, e.itemIndex);
		// add all listeners
		addAllListenersToAnItem(item);
		// add to array
		itemControls.add(e.itemIndex, item);
		// жǷӦù˵item
		if(filter != null)
			if(filter.match(model.getItem(e.itemIndex)))
				hideItem(e.itemIndex, true);
		// adjust SortData index of tabs which are after inserted tab
		int size = itemControls.size();
		for(int i = e.itemIndex + 1; i < size; i++)
			((SortData)getItemControl(i).getLayoutData()).index++;
		// content relayout
		content.layout();
		
		// relayout
		relayout();
		
		// call the model listeners
		int count = modelListeners.size();
		for(int i=0;i<count;i++)
			((ModelListener)modelListeners.get(i)).modelAdded(e);
	}

	/* (non-Javadoc)
	 * @see edu.tsinghua.swt.events.ModelListener#modelRemoved(edu.tsinghua.swt.events.ModelEvent)
	 */
	public void modelRemoved(ModelEvent e) {
		// dispose control which is associated with removed item
		Control item = (Control)itemControls.remove(e.itemIndex);
		item.dispose();
		
		// adjust SortData index of tabs which are after the tab deleted
		int size = itemControls.size();
		for(int i = e.itemIndex + 1; i < size; i++)
			((SortData)getItemControl(i).getLayoutData()).index--;
		
		// relayout
		relayout();
		
		// call the model listeners
		int count = modelListeners.size();
		for(int i=0;i<count;i++)
			((ModelListener)modelListeners.get(i)).modelRemoved(e);
	}

	/* (non-Javadoc)
	 * @see edu.tsinghua.swt.events.PropertyListener#propertyAdded(edu.tsinghua.swt.events.PropertyEvent)
	 */
	public void propertyAdded(PropertyEvent e) {
		String key = e.key;
		int index = model.indexOf(e.getSource());
		// -1˵nodeڣѾͷţ
		if(index == -1 || content.isDisposed() || isDisposed()) return;
		// nameimageıˣˢʾ
		if(key.equals("name")) 
			refreshItemName(index, (String)e.newValue);
		else if(key.equals("image"))
			refreshItemImage(index, (Image)e.newValue);
		
		// жǷӦù˵item
		if(filter != null)
			if(filter.match(model.getItem(index)))
				hideItem(index, true);
		// relayout if content layout is in sort mode
		if(((SortLayout)content.getLayout()).sort)
			content.layout();
		// call the property listeners
		int count = propertyListeners.size();
		for(int i=0;i<count;i++)
			((PropertyListener)propertyListeners.get(i)).propertyAdded(e);
	}

	/* (non-Javadoc)
	 * @see edu.tsinghua.swt.events.PropertyListener#propertyRemoved(edu.tsinghua.swt.events.PropertyEvent)
	 */
	public void propertyRemoved(PropertyEvent e) {
		String key = e.key;
		int index = model.indexOf(e.getSource());
		// -1˵nodeڣѾͷţ
		if(index == -1 || content.isDisposed() || isDisposed()) return;
		// nameimageıˣˢʾ
		if(key.equals("name")) 
			refreshItemName(index, "Noname");
		else if(key.equals("image"))
			refreshItemImage(index, null);
		
		// жǷӦù˵item
		if(filter != null)
			if(filter.match(model.getItem(index)))
				hideItem(index, true);
		// relayout if content layout is in sort mode
		if(((SortLayout)content.getLayout()).sort)
			content.layout();
		// call the property listeners
		int count = propertyListeners.size();
		for(int i=0;i<count;i++)
			((PropertyListener)propertyListeners.get(i)).propertyRemoved(e);
	}

	/* (non-Javadoc)
	 * @see edu.tsinghua.swt.events.PropertyListener#propertyModified(edu.tsinghua.swt.events.PropertyEvent)
	 */
	public void propertyModified(PropertyEvent e) {
		String key = e.key;
		int index = model.indexOf(e.getSource());
		// -1˵nodeڣѾͷţ
		if(index == -1 || content.isDisposed() || isDisposed()) return;
		// nameimageıˣˢʾ
		if(key.equals("name")) 
			refreshItemName(index, (String)e.newValue);
		else if(key.equals("image"))
			refreshItemImage(index, (Image)e.newValue);
		
		// жǷӦù˵item
		if(filter != null)
			if(filter.match(model.getItem(index)))
				hideItem(index, true);
		// relayout if content layout is in sort mode
		if(((SortLayout)content.getLayout()).sort)
			content.layout();
		// call the property listeners
		int count = propertyListeners.size();
		for(int i=0;i<count;i++)
			((PropertyListener)propertyListeners.get(i)).propertyModified(e);
	}
	
	// if item's name and has changed, refresh it
	private void refreshItemName(int index, String newName) {
		getItemControl(index).setText(newName);
	}
	
	// if item's image has changed, refresh it
	private void refreshItemImage(int index, Image newImage) {
		CoolButton item = getItemControl(index);
		item.setImage(newImage);
		item.setShowSmallImage(showSmall);
	}
	
	/**
	 * edit name of an item which is specified by itemIndex
	 * @param itemIndex item index
	 */
	public void editItemName(int itemIndex) {
		if(itemIndex < 0 || itemIndex >= itemControls.size()) return;
		CoolButton item = (CoolButton)itemControls.get(itemIndex);
		item.editText();
	}

	/* (non-Javadoc)
	 * @see edu.tsinghua.swt.events.CoolButtonListener#textChanged(edu.tsinghua.swt.events.CoolButtonEvent)
	 */
	public void textChanged(CoolButtonEvent e) {
		// modify layout sort field
		CoolButton item = (CoolButton)e.getSource();
		// modify model
		model.setItemName(indexOf(item), e.newText);
		// relayout if current layout is in sort mode
		if(((SortLayout)content.getLayout()).sort)
			content.layout();
	}
	
	/**
	 * sort items
	 * @param ascendent
	 */
	public void sortItems(boolean ascendent) {
		if(content == null) return;
		SortLayout layout = (SortLayout)content.getLayout();
		if(!layout.sort) {
			layout.sort = true;
			layout.ascendent = ascendent;
			content.layout();
		} else if(layout.ascendent != ascendent) {
			layout.ascendent = ascendent;
			content.layout();
		}
	}
	
	/**
	 * unsort items
	 */
	public void unsortItems() {
		if(content == null) return;
		SortLayout layout = (SortLayout)content.getLayout();
		if(layout.sort) {
			layout.sort = false;
			content.layout();
		}
	}
	
	/**
	 * ƶitemʱʾɫ
	 * @param color ɫ
	 */
	public void setMouseTrackColor(Color color) {
		if(color == null) return;
		this.mouseTrackColor = color;
		int size = itemControls.size();
		for(int i = 0; i < size; i++)
			((CoolButton)itemControls.get(i)).setMouseTrackColor(mouseTrackColor);
	}
	
	/**
	 * ûԶĸӵExtraWorkӿʵ
	 * @param extra IExtraWork
	 */
	public void setExtraWork(IExtraWork extra) {
		this.extraWork = extra;
	}
	
	/**
	 * һitem
	 * @param itemIndex Ҫصitemindex
	 * @param layout trueʾlayoutfalseʾlayoutһض
	 * 		   Itemʱһlayout
	 */
	public void hideItem(int itemIndex, boolean layout) {
		_hideItem(itemIndex);
		if(layout) {
			this.layout();
			content.layout();
		}
	}
	
	// ҪΪ֮ȲlayoutΪûù˶ÿlayout
	//     һҾñȽûЧʣ԰ⲿִŪ
	private void _hideItem(int itemIndex) {
		CoolButton item = getItemControl(itemIndex);
		if(item == null) return;
		((SortData)item.getLayoutData()).heightHint = 0;
		item.setVisible(false);
	}
	
	/**
	 * ʾһitemǿӵģЧ
	 * @param itemIndex
	 * @param layout trueʾlayoutfalseʾlayoutһض
	 * 		   Itemʱһlayout
	 */
	public void showItem(int itemIndex, boolean layout) {
		CoolButton item = getItemControl(itemIndex);
		if(item == null) return;
		if(item.getVisible()) return;
		((SortData)item.getLayoutData()).heightHint = SWT.DEFAULT;
		item.setVisible(true);
		if(layout) {
			this.layout();
			content.layout();
		}
	}
	
	/**
	 * itemûṩһMatcherʵ֣ڷtrueֵitemʾ
	 * @param filter
	 */
	public void filter(IMatcher filter) {
		// filter
		this.filter = filter;
		// itemΪ
		int size = itemControls.size();
		for(int i = 0; i < size; i++) 
			getItemControl(i).setVisible(true);
		// طitem
		int[] matches = model.findItemIndex(filter);
		if(matches == null) return;
		for(int i = 0; i < matches.length; i++)
			_hideItem(matches[i]);
		// relayout
		content.layout();
	}
	
	/**
	 * @return Returns the showSmall.
	 */
	public boolean isShowSmallImage() {
		return showSmall;
	}
	
	/**
	 * @return Returns the showText.
	 */
	public boolean isShowText() {
		return showText;
	}
	
	/**
	 * @param contentAlignment The contentAlignment to set.
	 */
	public void setContentAlignment(int contentAlignment) {
		this.contentAlignment = contentAlignment;
		if(content != null) {
			if(contentAlignment == SWT.LEFT)
				content.setLocation(0, contentY);
			else if(contentAlignment == SWT.RIGHT)
				content.setLocation(this.getSize().x - content.getSize().x, contentY);
			else if(contentAlignment == SWT.CENTER)
				content.setLocation((this.getSize().x - content.getSize().x) / 2, contentY);
		}
	}
	
	/**
	 * ˢͼ
	 */
	public void refreshView() {
		relayout();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.swt.widgets.Control#computeSize(int, int, boolean)
	 */
	public Point computeSize(int widthHint, int heightHint, boolean flushCache) {
		Point size = super.computeSize(widthHint, heightHint, flushCache);
		if(widthHint != -1)
			size.x = widthHint;
		if(heightHint != -1)
			size.y = heightHint;
		return size;
	}
	
	/**
	 * itemƵĵһĸߵһֵĸλitemλ
	 * 
	 * @param c
	 * 			һĸ
	 */
	public void revealItem(char c) {
	    int contentX = content.getLocation().x;
	    
	    // пͷĸĸcitem
	    firstMatcher.setCharacter(c);
	    int[] index = model.findItemIndex(firstMatcher);
	    if(index == null || index.length == 0) return;
	    
	    // itemy˳
	    int[] y = generateYArray(index);
	    Arrays.sort(y);
	    
	    // ʾһitem
	    for(int i = 0; i < y.length; i++) {
	        if(y[i] > -contentY) {
	            contentY = -y[i];
	            content.setLocation(contentX, contentY);
	            adjustButtonVisibility();
	            return;
	        }
	    }
	    
	    // е˵ĿǰʾһitemforûгҪʾһitem
	    contentY = -y[0];
        content.setLocation(contentX, contentY);
        adjustButtonVisibility();
	}
	
	/**
	 * еitemy
	 * 
	 * @param index
	 * 		item
	 * @return itemy飬ycontent
	 */
	private int[] generateYArray(int[] index) {
	    int[] y = new int[index.length];
	    for(int i = 0; i < index.length; i++) {
	        CoolButton cb = getItemControl(index[i]);
	        y[i] = cb.getLocation().y;
	    }
	    return y;
	}
	
	/**
	 * ʾindexitem
	 * 
	 * @param index
	 * 			item
	 */
	public void revealItem(int index) {
	    CoolButton cb = getItemControl(index);
	    if(cb != null) {
	        Point p = cb.getLocation();
	        contentY = -p.y;
	        content.setLocation(content.getLocation().x, contentY);
	        adjustButtonVisibility();
	    }
	}
}
