/*
* Shutter - Friend List Component
*
* Copyright (C) 2004 luma <stubma@163.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
package edu.tsinghua.swt.widgets;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.Iterator;

import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DragSourceListener;
import org.eclipse.swt.dnd.DropTargetListener;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.MouseMoveListener;
import org.eclipse.swt.events.MouseTrackListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import edu.tsinghua.swt.events.*;
import edu.tsinghua.swt.internal.events.TabMouseListener;
import edu.tsinghua.swt.internal.layout.DefaultNodeComparator;
import edu.tsinghua.swt.internal.resources.Icons;
import edu.tsinghua.swt.layout.SortData;
import edu.tsinghua.swt.layout.SortLayout;
import edu.tsinghua.swt.models.*;

/**
 * shutter is a custom widget likes the one in QQ, I call it "Shutter"
 * @author luma@smth
 */
public class Shutter extends Composite implements ShutterModelListener, DisposeListener, PropertyListener, CoolButtonListener {	
	// shutter's model
	private ShutterModel model;
	// current tab
	private int currentTab;
	// tab's control array
	private ArrayList tabControls;
	// ScrollList array, corresponding to tabControls
	private ArrayList scrollLists;
	// current display
	private Display display;
	// tab mouse listeners
	private ArrayList tabMouseListeners;
	// tab mouse move listeners
	private ArrayList tabMouseMoveListeners;
	// tab mouse track listeners
	private ArrayList tabMouseTrackListeners;
	// tab drag source listeners
	private ArrayList tabDragSourceListeners;
	// tab drop target listeners
	private ArrayList tabDropTargetListeners;
	// item drag source listeners
	private ArrayList itemDragSourceListeners;
	// item drop target listeners
	private ArrayList itemDropTargetListeners;
	// model listeners and property listeners
	private ArrayList modelListeners;
	private ArrayList propertyListeners;
	// shutter event listener
	private ArrayList shutterListeners;
	// default scroll list background color
	private Color defaultColor;
	// default tab cursor
	private Cursor handCursor;
	// ƶtabϵɫ
	private Color tabMouseTrackColor;
	// ƶitemϵɫ
	private Color itemMouseTrackColor;
	// ȱʡtab sorteritem sorter
	private Comparator itemSorter;
	// tab and item IExtraWork
	private IExtraWork tabExtra, itemExtra;
	// ǷItem
	private boolean itemSorted;
	// tabitemĹ
	private IMatcher tabFilter, itemFilter;
	// ǷʾСͼ
	private boolean showSmall;
	// Ƿʾ
	private boolean showText;
	// ֳ
	private int textOrientation;
	// ɫ
	private Color contentBackground;
	// content alignment
	private int contentAlignment;
	
	/**
	 * call super constructor
	 * @param parent
	 * @param style
	 */
	public Shutter(Composite parent, int style) {
		super(parent, style);
		
		// set shutter layout
		SortLayout sl = new SortLayout();
		sl.horizontalSpacing = sl.verticalSpacing = 0;
		sl.marginHeight = sl.marginWidth = 0;
		sl.keyComparator = new DefaultNodeComparator();
		setLayout(sl);
		
		// initialize some variable
		currentTab = -1;
		tabControls = new ArrayList();
		scrollLists = new ArrayList();
		display = getDisplay();
		tabMouseListeners = new ArrayList();
		tabMouseMoveListeners = new ArrayList();
		tabMouseTrackListeners = new ArrayList();
		tabDragSourceListeners = new ArrayList();
		tabDropTargetListeners = new ArrayList();
		itemDragSourceListeners = new ArrayList();
		itemDropTargetListeners = new ArrayList();
		modelListeners = new ArrayList();
		propertyListeners = new ArrayList();
		shutterListeners = new ArrayList();
		defaultColor = new Color(getDisplay(), 0x84, 0x82, 0xC6);
		handCursor = new Cursor(getDisplay(), SWT.CURSOR_HAND);
		itemSorter = new DefaultNodeComparator();
		itemSorted = false;
		showSmall = true;
		showText = true;
		textOrientation = SWT.RIGHT;
		contentBackground = null;
		contentAlignment = SWT.LEFT;
		
		// add itself as some type listener
		addDisposeListener(this);
		
		// initialize icons
		Icons.getInstance().loadResources();
	}
	
	/**
	 * set model
	 * @param model scroll list model
	 */
	public void setModel(ShutterModel model) {
		// if shutter has a mode, release it, and remove this from model listener
		if(this.model != null) {
			this.model.removeModelListener(this);
			int size = tabControls.size();
			for(int i = 0; i < size; i++) {
				((CoolButton)tabControls.get(i)).dispose();
				((ScrollList)scrollLists.get(i)).dispose();
			}
			tabControls.clear();
			scrollLists.clear();
		}
		// set scroll list model to this new one
		this.model = model;
		// add me as some listener types
		this.model.addModelListener(this);
		this.model.addPropertyListener(this);
		// reset current tab index to -1
		currentTab = -1;
		// initialize controls
		initControls();
		// relayout
		this.layout();
	}
	
	/**
	 * get shutter model
	 * @return a ShutterModel object associated with shutter
	 */
	public ShutterModel getModel() {
		return model;
	}
	
	// create controls and initialize theirs' state
	private void initControls() {
		if(model != null) {
			// create controls
			createControlsFromModel();
			
			// add all listeners to tabs
			addAllListenersToAllTabs();
			
			// if the number of tab is larger than 0, show tab 0 as current tab
			if(tabControls.size() > 0)
				setCurrentTab(0);
		}
	}
	
	/**
	 * ǰӵlistenerµmodelǲʹǰõlistenerʱ
	 * ִи÷
	 */
	public void clearListeners() {
		tabMouseListeners.clear();
		tabMouseMoveListeners.clear();
		tabMouseTrackListeners.clear();
		tabDragSourceListeners.clear();
		tabDropTargetListeners.clear();
	}

	// add all listeners to all tabs
	private void addAllListenersToAllTabs() {
		Iterator iter = tabControls.iterator();
		while(iter.hasNext())
			addAllListenersToATab((CoolButton)iter.next());
	}
	
	// add all listeners to a tab
	private void addAllListenersToATab(CoolButton tab) {
		// add mouse track listener
		Iterator iter = tabMouseTrackListeners.iterator();
		while(iter.hasNext())
			tab.addMouseTrackListener((MouseTrackListener)iter.next());
		// add mouse move listener
		iter = tabMouseMoveListeners.iterator();
		while(iter.hasNext())
			tab.addMouseMoveListener((MouseMoveListener)iter.next());
		// add mouse listener
		iter = tabMouseListeners.iterator();
		while(iter.hasNext())
			tab.addMouseListener((MouseListener)iter.next());
		// add drag source listener
		iter = tabDragSourceListeners.iterator();
		while(iter.hasNext())
			tab.addDragSourceListener((DragSourceListener)iter.next());
		// add drop target listener
		iter = tabDropTargetListeners.iterator();
		while(iter.hasNext())
			tab.addDropTargetListener((DropTargetListener)iter.next());
	}

	// create controls from model
	private void createControlsFromModel() {		
		// get tab count
		int count = model.getTabCount();
		// create all controls in loop
		for(int i=0;i<count;i++) {
			// create tab
			CoolButton tab = new CoolButton(this, MySWT.HOVER | SWT.CENTER, model.getTabName(i), model.getTabImage(i));
			// initialize tab
			initTab(tab, i);
			// add default tab mouse listener
			tab.addMouseListener(new TabMouseListener(this));
			// add this as shutter label listener
			tab.addShutterLabelListener(this);
			// add to array
			tabControls.add(tab);
			
			// create a scroll list corresponding to tab
			ScrollList sl = new ScrollList(this, SWT.NONE);			
			// initialize scroll list
			initScrollList(sl, model.getTabModel(i), tab);
			// add scroll list to array
			scrollLists.add(sl);			
			// жǷӦù˵tab
			if(tabFilter != null)
				if(tabFilter.match(model.getTab(i)))
					hideTab(i, true);
		}
	}
	
	// ½һScrollListʱԽгʼ
	private void initScrollList(ScrollList sl, ScrollListModel model, CoolButton tab) {
		sl.setContent(new Composite(sl, SWT.NONE));
		// set scroll list model
		sl.setModel(model);
		// set backgroud
		if(contentBackground != null)
			sl.setContentBackground(contentBackground);
		else
			sl.setContentBackground(defaultColor);
		// set other presentation option
		sl.setShowSmallImage(showSmall);
		sl.setShowText(showText);
		sl.setTextOrientation(textOrientation);
		sl.setContentAlignment(contentAlignment);
		// set item mouse track color
		if(itemMouseTrackColor != null) sl.setMouseTrackColor(itemMouseTrackColor);
		// set item sorter
		sl.setSorter(itemSorter);
		if(itemSorted)
			sl.sortItems(true);
		// set item extra
		sl.setExtraWork(itemExtra);
		// add item filter to scroll list
		if(itemFilter != null)
			sl.filter(itemFilter);
		// set scroll layout data
		//    at the begining, scroll lists' height is all 0
		SortData sd = new SortData(SortData.FILL_HORIZONTAL, tab);
		sd.heightHint = 0;
		sl.setLayoutData(sd);
		// ShutterScrollListԼΪitem¼ɷscroll listģ
		//     ԼΪղ¼
		sl.addPropertyListener(this);
		// 
		int size = itemDragSourceListeners.size();
		for(int i = 0; i < size; i++)
			sl.addItemDragSourceListener((DragSourceListener)itemDragSourceListeners.get(i));
		 size = itemDropTargetListeners.size();
		for(int i = 0; i < size; i++)
			sl.addItemDropTargetListener((DropTargetListener)itemDropTargetListeners.get(i));
	}
	
	/**
	 * get tab control associated with a tab
	 * @param index tab index
	 * @return get tab control which is a CoolButton
	 */
	public CoolButton getTabControl(int index) {
		if(index < 0 || index >= tabControls.size()) return null;
		return (CoolButton)tabControls.get(index);
	}
	
	/**
	 * غnodeӦcontrolҲǸmodelѰӦview
	 * @param node
	 * @return غnodeӦcontrol
	 */
	public CoolButton getTabControl(INode node) {
		return getTabControl(model.indexOf(node));
	}
	
	/**
	 * get scroll list control associated with a tab
	 * @param index tab index
	 * @return get ScrollList object which is specified by index 
	 */
	public ScrollList getScrollList(int index) {
		if(index < 0 || index >= scrollLists.size()) return null;
		return (ScrollList)scrollLists.get(index);
	}
	
	/**
	 * return tab index
	 * @param tab tab control
	 * @return the tab control's index, the returned index specify the control position in
	 * 		internal array, so even the tabs is sorted, the index is also valid and can be 
	 * 		passed to getTabControl to get control
	 */
	public int indexOf(Control tab) {
		return tabControls.indexOf(tab);
	}
	
	/**
	 * facade method, get an control associated with an item which is specified
	 * by tabIndex and itemIndex
	 * @param tabIndex tab index
	 * @param itemIndex item index
	 * @return get item control which is a CoolButton object
	 */
	public CoolButton getItemControl(int tabIndex, int itemIndex) {
		if(tabIndex < 0 || itemIndex < 0 || tabIndex >= tabControls.size()) return null;
		return ((ScrollList)scrollLists.get(tabIndex)).getItemControl(itemIndex);
	}
	
	/**
	 * غnodeӦcontrol
	 * @param node
	 * @return
	 */
	public CoolButton getItemControl(INode node) {
		int size = scrollLists.size();
		for(int i = 0; i < size; i++) {
			CoolButton ret = ((ScrollList)scrollLists.get(i)).getItemControl(node);
			if(ret != null) return ret;
		}
		return null;
	}
	
	/**
	 * facade method, add an item to a tab which is specified by tabIndex
	 * @param tabIndex tab index
	 * @param item GeneralItem object
	 */
	public void addItem(int tabIndex, INode item) {
		if(tabIndex < 0 || tabIndex >= tabControls.size()) return;
		model.addItem(tabIndex, item);
	}
	
	/**
	 * @return current tab index
	 */
	public int getCurrentTab() {
		return currentTab;
	}
	
	/**
	 * set current tab
	 * @param tabIndex
	 */
	public void setCurrentTab(int tabIndex) {
		if(tabIndex != currentTab && tabIndex >= 0 && tabIndex < tabControls.size()) {
			// set the scrollList specified by tabIndex to appropriate size
			ScrollList sl = (ScrollList)scrollLists.get(tabIndex);
			SortData sd = (SortData)sl.getLayoutData();
			sd.heightHint = SWT.DEFAULT;
			sd.grabExcessVerticalSpace = true;
			sd.verticalAlignment = SortData.FILL;
			// if current tab is -1, then no need set current tab to Zero
			if(currentTab != -1) {
				sl = (ScrollList)scrollLists.get(currentTab);
				sd = (SortData)sl.getLayoutData();
				sd.heightHint = 0;
				sd.grabExcessVerticalSpace = false;
				sd.verticalAlignment = SortData.BEGINNING;
			}
			// generate event object
			ShutterEvent event = new ShutterEvent(this);
			event.oldTab = currentTab;
			event.newTab = tabIndex;
			// set current tab index
			currentTab = tabIndex;
			// relayout
			this.layout();
			// call shutter listener
			int count = shutterListeners.size();
			for(int i=0;i<count;i++)
				((ShutterListener)shutterListeners.get(i)).tabSwitched(event);
		}
	}
	
	/**
	 * TabϷԴ
	 * @param listener
	 */
	public void addTabDragSourceListener(DragSourceListener listener) {
		if(listener != null) {
			Iterator iter = tabControls.iterator();
			while(iter.hasNext())
				((CoolButton)iter.next()).addDragSourceListener(listener);
			tabDragSourceListeners.add(listener);
		}
	}
	
	/**
	 * ɾTabϷԴ
	 * @param listener
	 */
	public void removeTabDragSourceListener(DragSourceListener listener) {
		if(listener != null) {
			Iterator iter = tabControls.iterator();
			while(iter.hasNext())
				((CoolButton)iter.next()).removeDragSourceListener(listener);
			tabDragSourceListeners.remove(listener);
		}
	}
	
	/**
	 * TabϷĿļ
	 * @param listener
	 */
	public void addTabDropTargetListener(DropTargetListener listener) {
		if(listener != null) {
			Iterator iter = tabControls.iterator();
			while(iter.hasNext())
				((CoolButton)iter.next()).addDropTargetListener(listener);
			tabDropTargetListeners.add(listener);
		}
	}	
	
	/**
	 * ɾTabϷĿļ
	 * @param listener
	 */
	public void removeTabDropTargetListener(DropTargetListener listener) {
		if(listener != null) {
			Iterator iter = tabControls.iterator();
			while(iter.hasNext())
				((CoolButton)iter.next()).removeDropTargetListener(listener);
			tabDropTargetListeners.remove(listener);
		}
	}	
	
	/**
	 * itemϷԴ
	 * @param listener
	 */
	public void addItemDragSourceListener(DragSourceListener listener) {
		itemDragSourceListeners.add(listener);
		int size = tabControls.size();
		for(int i = 0; i < size; i++)
			((ScrollList)scrollLists.get(i)).addItemDragSourceListener(listener);
	}
	
	/**
	 * ɾitemϷԴ
	 * @param listener
	 */
	public void removeItemDragSourceListener(DragSourceListener listener) {
		itemDragSourceListeners.remove(listener);
		int size = tabControls.size();
		for(int i = 0; i < size; i++)
			((ScrollList)scrollLists.get(i)).removeItemDragSourceListener(listener);
	}
	
	/**
	 * itemϷĿļ
	 * @param listener
	 */
	public void addItemDropTargetListener(DropTargetListener listener) {
		itemDropTargetListeners.add(listener);
		int size = tabControls.size();
		for(int i = 0; i < size; i++)
			((ScrollList)scrollLists.get(i)).addItemDropTargetListener(listener);
	}
	
	/**
	 * ɾitemϷĿļ
	 * @param listener
	 */
	public void removeItemDropTargetListener(DropTargetListener listener) {
		itemDropTargetListeners.remove(listener);
		int size = tabControls.size();
		for(int i = 0; i < size; i++)
			((ScrollList)scrollLists.get(i)).removeItemDropTargetListener(listener);
	}
	
	/**
	 * add tabs' mouse track listener which will be invoked when mouse enter or exit an tab
	 * @param listener
	 */
	public void addTabMouseTrackListener(MouseTrackListener listener) {
		if(listener != null) {
			Iterator iter = tabControls.iterator();
			while(iter.hasNext())
				((CoolButton)iter.next()).addMouseTrackListener(listener);
			tabMouseTrackListeners.add(listener);
		}
	}
	
	/**
	 * remove tabs' mouse track listener
	 * @param listener
	 */
	public void removeTabMouseTrackListener(MouseTrackListener listener) {
		if(listener != null) {
			Iterator iter = tabControls.iterator();
			while(iter.hasNext())
				((CoolButton)iter.next()).removeMouseTrackListener(listener);
			tabMouseTrackListeners.remove(listener);
		}
	}
	
	/**
	 * add tabs' mouse move listener which will be invoked when mouse pass an tab
	 * @param listener
	 */
	public void addTabMouseMoveListener(MouseMoveListener listener) {
		if(listener != null) {
			Iterator iter = tabControls.iterator();
			while(iter.hasNext())
				((CoolButton)iter.next()).addMouseMoveListener(listener);
			tabMouseMoveListeners.add(listener);
		}
	}
	
	/**
	 * remove tabs' mouse move listener
	 * @param listener
	 */
	public void removeTabMouseMoveListener(MouseMoveListener listener) {
		if(listener != null) {
			Iterator iter = tabControls.iterator();
			while(iter.hasNext())
				((CoolButton)iter.next()).removeMouseMoveListener(listener);
			tabMouseMoveListeners.remove(listener);
		}
	}
	
	/**
	 * add tabs' mouse listener which will be invoked when mouse click on an tab
	 * @param listener
	 */
	public void addTabMouseListener(MouseListener listener) {
		if(listener != null) {
			Iterator iter = tabControls.iterator();
			while(iter.hasNext())
				((CoolButton)iter.next()).addMouseListener(listener);
			tabMouseListeners.add(listener);
		}
	}
	
	/**
	 * remove tabs' mouse listener
	 * @param listener
	 */
	public void removeTabMouseListener(MouseListener listener) {
		if(listener != null) {
			Iterator iter = tabControls.iterator();
			while(iter.hasNext())
				((CoolButton)iter.next()).removeMouseListener(listener);
			tabMouseListeners.remove(listener);
		}
	}
	
	/**
	 * facade method, set content columns in a tab
	 * @param cols columns
	 * @param tabIndex tabIndex tabIndex The index of tab whose columns should be changed. 
	 * 		  If it's MySWT.ALL, change all tabs
	 */
	public void setContentColumns(int cols, int tabIndex) {
	    if(tabIndex == MySWT.ALL) {
	        Iterator iter = scrollLists.iterator();
	        while(iter.hasNext())
	            ((ScrollList)iter.next()).setContentColumns(cols);
	    } else {
			if(tabIndex < 0 || tabIndex >= tabControls.size()) return;
			((ScrollList)scrollLists.get(tabIndex)).setContentColumns(cols);	        
	    }
	}
	
	/**
	 * facade method, set whether show small image or not in a tab
	 * @param small true if show small image, otherwise false
	 */
	public void setShowSmallImage(boolean small) {
		this.showSmall = small;
        Iterator iter = scrollLists.iterator();
        while(iter.hasNext())
            ((ScrollList)iter.next()).setShowSmallImage(small);
	}
	
	/**
	 * facade method, set whether show text or not in a tab
	 * @param show true if show, otherwise false
	 */
	public void setShowText(boolean show) {
		this.showText = show;
        Iterator iter = scrollLists.iterator();
        while(iter.hasNext())
            ((ScrollList)iter.next()).setShowText(show);
	}
	
	/**
	 * facade method, set item text orientaion in a tab
	 * @param orientation
	 */
	public void setTextOrientation(int orientation) {
		this.textOrientation = orientation;
        Iterator iter = scrollLists.iterator();
        while(iter.hasNext())
            ((ScrollList)iter.next()).setTextOrientation(orientation);
	}
	
	/**
	 * facade method, set current tab's content background
	 * @param color background color
	 */
	public void setContentBackground(Color color) {
		this.contentBackground = color;
        Iterator iter = scrollLists.iterator();
        while(iter.hasNext())
        	((ScrollList)iter.next()).setContentBackground(color);            
	}
	
	/**
	 * facade method, set an item's tooltip text
	 * @param text
	 * @param tabIndex The index of tab whose item's tooltip should be set. 
	 * 		  If it's MySWT.ALL, set all tabs
	 * @param itemIndex The index of item whose tooltip should be set. 
	 * 		  If it's MySWT.ALL, set all items in a tab
	 */
	public void setItemToolTipText(String text, int tabIndex, int itemIndex) {
	    if(tabIndex == MySWT.ALL) {
	        Iterator iter = scrollLists.iterator();
	        while(iter.hasNext())
	            ((ScrollList)iter.next()).setToolTipText(text, itemIndex);
	    } else {
			if(tabIndex < 0 || tabIndex >= tabControls.size()) return;
			((ScrollList)scrollLists.get(tabIndex)).setToolTipText(text, itemIndex);	        
	    }
	}
	
	/**
	 * add a model listener
	 * @param listener
	 */
	public void addModelListener(ShutterModelListener listener) {
		modelListeners.add(listener);
	}
	
	/**
	 * remove a model listener
	 * @param listener
	 */
	public void removeModelListener(ShutterModelListener listener) {
		modelListeners.remove(listener);
	}
	
	/**
	 * add a property listener
	 * @param listener
	 */
	public void addPropertyListener(PropertyListener listener) {
		propertyListeners.add(listener);
	}
	
	/**
	 * remove a property listener
	 * @param listener
	 */
	public void removePropertyListener(PropertyListener listener) {
		propertyListeners.remove(listener);
	}
	
	/**
	 * add a shutter event listener
	 * @param listener
	 */
	public void addShutterListener(ShutterListener listener) {
		shutterListeners.add(listener);
	}
	
	/**
	 * remove a shutter listener
	 * @param listener
	 */
	public void removeShutterListener(ShutterListener listener) {
		shutterListeners.remove(listener);
	}
	
	/**
	 * content˵ҲǺѿؼ֮¼
	 * @param listener
	 * @param tabIndex
	 */
	public void addContentMouseListener(MouseListener listener, int tabIndex) {
		if(tabIndex < 0 || tabIndex >= tabControls.size()) return;
		((ScrollList)scrollLists.get(tabIndex)).addMouseListener(listener);
	}
	
	/**
	 * ɾcontent˵ҲǺѿؼ֮¼
	 * @param listener
	 * @param tabIndex
	 */
	public void removeContentMouseListener(MouseListener listener, int tabIndex) {
		if(tabIndex < 0 || tabIndex >= tabControls.size()) return;
		((ScrollList)scrollLists.get(tabIndex)).removeMouseListener(listener);
	}	
	
	/**
	 * facade method, add a content mouse move listener in a tab
	 * @param listener mouse listener
	 * @param tabIndex tab index
	 */
	public void addContentMouseMoveListener(MouseMoveListener listener, int tabIndex) {
		if(tabIndex < 0 || tabIndex >= tabControls.size()) return;
		((ScrollList)scrollLists.get(tabIndex)).addMouseMoveListener(listener);
	}
	
	/**
	 * facade method, rmove a content mouse move listener in a tab
	 * @param listener mouse listener
	 * @param tabIndex tab index
	 */
	public void removeContentMouseMoveListener(MouseMoveListener listener, int tabIndex) {
		if(tabIndex < 0 || tabIndex >= tabControls.size()) return;
		((ScrollList)scrollLists.get(tabIndex)).removeMouseMoveListener(listener);
	}
	
	/**
	 * facade method, add a content mouse track listener in a tab
	 * @param listener mouse listener
	 * @param tabIndex tab index
	 */
	public void addContentMouseTrackListener(MouseTrackListener listener, int tabIndex) {
		if(tabIndex < 0 || tabIndex >= tabControls.size()) return;
		((ScrollList)scrollLists.get(tabIndex)).addMouseTrackListener(listener);
	}
	
	/**
	 * facade method, remove a content mouse track listener in a tab
	 * @param listener mouse listener
	 * @param tabIndex tab index
	 */
	public void removeContentMouseTrackListener(MouseTrackListener listener, int tabIndex) {
		if(tabIndex < 0 || tabIndex >= tabControls.size()) return;
		((ScrollList)scrollLists.get(tabIndex)).removeMouseTrackListener(listener);
	}
	
	/**
	 * facade method, add a mouse listener to all item in a tab
	 * @param listener mouse listener
	 * @param tabIndex tab index
	 */
	public void addItemMouseListener(MouseListener listener, int tabIndex) {
		if(tabIndex < 0 || tabIndex >= tabControls.size()) return;
		((ScrollList)scrollLists.get(tabIndex)).addItemMouseListener(listener);
	}
	
	/**
	 * facade method, remove a mouse listener from all item in a tab
	 * @param listener mouse listener
	 * @param tabIndex tab index
	 */
	public void removeItemMouseListener(MouseListener listener, int tabIndex) {
		if(tabIndex < 0 || tabIndex >= tabControls.size()) return;
		((ScrollList)scrollLists.get(tabIndex)).removeItemMouseListener(listener);
	}
	
	/**
	 * facade method, add a mouse move listener to all item in a tab
	 * @param listener mouse listener
	 * @param tabIndex tab index
	 */
	public void addItemMouseMoveListener(MouseMoveListener listener, int tabIndex) {
		if(tabIndex < 0 || tabIndex >= tabControls.size()) return;
		((ScrollList)scrollLists.get(tabIndex)).addItemMouseMoveListener(listener);
	}
	
	/**
	 * facade method, remove a mouse move listener from all item in a tab
	 * @param listener mouse listener
	 * @param tabIndex tab index
	 */
	public void removeItemMouseMoveListener(MouseMoveListener listener, int tabIndex) {
		if(tabIndex < 0 || tabIndex >= tabControls.size()) return;
		((ScrollList)scrollLists.get(tabIndex)).removeItemMouseMoveListener(listener);
	}
	
	/**
	 * facade method, add a mouse track listener to all item in a tab
	 * @param listener mouse listener
	 * @param tabIndex tab index
	 */
	public void addItemMouseTrackListener(MouseTrackListener listener, int tabIndex) {
		if(tabIndex < 0 || tabIndex >= tabControls.size()) return;
		((ScrollList)scrollLists.get(tabIndex)).addItemMouseTrackListener(listener);
	}
	
	/**
	 * facade method, remove a mouse track listener from all item in a tab
	 * @param listener mouse listener
	 * @param tabIndex tab index
	 */
	public void removeItemMouseTrackListener(MouseTrackListener listener, int tabIndex) {
		if(tabIndex < 0 || tabIndex >= tabControls.size()) return;
		((ScrollList)scrollLists.get(tabIndex)).removeItemMouseTrackListener(listener);
	}
	
	/**
	 * start blink an image on a tab which is specified by tabIndex
	 * @param blinkImage image to be blinked
	 * @param tabIndex tab index
	 */
	public void startBlinkImage(Image blinkImage, int tabIndex) {
		if(tabIndex < 0 || tabIndex >= tabControls.size()) return;
		((CoolButton)tabControls.get(tabIndex)).startBlinkImage(blinkImage);
	}
	
	/**
	 * stop blink image on a tab which is specified by tabIndex
	 * @param tabIndex tab index
	 */
	public void stopBlinkImage(int tabIndex) {
		if(tabIndex < 0 || tabIndex >= tabControls.size()) return;
		((CoolButton)tabControls.get(tabIndex)).stopBlinkImage();		
	}
	
	/**
	 * start leap an image on a tab which is specified by tabIndex
	 * @param leapImage image to be leaped
	 * @param tabIndex tab index
	 */
	public void startBounceImage(Image leapImage, int tabIndex) {
		if(tabIndex < 0 || tabIndex >= tabControls.size()) return;
		((CoolButton)tabControls.get(tabIndex)).startBounceImage(leapImage);		
	}
	
	/**
	 * stop leaping image
	 * @param tabIndex tab index
	 */
	public void stopBounceImage(int tabIndex) {
		if(tabIndex < 0 || tabIndex >= tabControls.size()) return;
		((CoolButton)tabControls.get(tabIndex)).stopBounceImage();		
	}
	
	/**
	 * edit name of a tab which is specified by tabIndex
	 * @param tabIndex tab index
	 */
	public void editTabName(int tabIndex) {
		if(tabIndex < 0 || tabIndex >= tabControls.size()) return;
		CoolButton tab = (CoolButton)tabControls.get(tabIndex);
		tab.editText();
	}
	
	/**
	 * edit name of an item which is specified by tabIndex and itemIndex
	 * @param tabIndex
	 * @param itemIndex
	 */
	public void editItemName(int tabIndex, int itemIndex) {
		if(tabIndex < 0 || tabIndex >= tabControls.size()) return;
		((ScrollList)scrollLists.get(tabIndex)).editItemName(itemIndex);
	}
	
	/**
	 * sort tab
	 * @param ascendent sort order
	 */
	public void sortTabs(boolean ascendent) {
		SortLayout layout = (SortLayout)getLayout();
		if(!layout.sort) {
			layout.sort = true;
			layout.ascendent = ascendent;
			this.layout();
		} else if(layout.ascendent != ascendent) {
			layout.ascendent = ascendent;
			this.layout();
		}
	}
	
	/**
	 * sort items in one tab which is specified by tabIndex
	 * @param tabIndex
	 * @param ascendent
	 */
	public void sortItems(boolean ascendent) {
		itemSorted = true;
		int size = scrollLists.size();
		for(int i = 0; i < size; i++)
			((ScrollList)scrollLists.get(i)).sortItems(ascendent);
	}
	
	/**
	 * unsort tab
	 */
	public void unsortTabs() {
		SortLayout layout = (SortLayout)getLayout();
		if(layout.sort) {
			layout.sort = false;
			this.layout();
		}
	}
	
	public void unsortItems() {
		itemSorted = false;
		int size = scrollLists.size();
		for(int i = 0; i < size; i++)
			((ScrollList)scrollLists.get(i)).unsortItems();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.DisposeListener#widgetDisposed(org.eclipse.swt.events.DisposeEvent)
	 */
	public void widgetDisposed(DisposeEvent e) {		
		// free resources
		Icons.getInstance().freeResources();
		defaultColor.dispose();
		handCursor.dispose();
	}
	
	// ڴһTabʱʼ
	private void initTab(CoolButton tab, int index) {
		// set layout data
		SortData sd = new SortData(SortData.FILL_HORIZONTAL, index, model.getTab(index));
		tab.setLayoutData(sd);
		// add model as data
		tab.setData(model.getTab(index));
		// set default tab cursor
		tab.setCursor(handCursor);
		// set tab mouse track color
		if(tabMouseTrackColor != null) tab.setMouseTrackColor(tabMouseTrackColor);
		// do extra work if there is any
		if(tabExtra != null)
			tabExtra.doExtraWork(tab, model.getTab(index));
	}

	/* (non-Javadoc)
	 * @see edu.tsinghua.swt.events.ModelListener#modelAdded(edu.tsinghua.swt.events.ModelEvent)
	 */
	public void modelAdded(ModelEvent e) {
		ShutterModelEvent event = (ShutterModelEvent)e;
		int tabIndex = event.tabIndex;
		int itemIndex = event.itemIndex;
		ShutterModel m = (ShutterModel)event.getSource();
		
		// create tab
		CoolButton tab = new CoolButton(this, MySWT.HOVER | SWT.CENTER, model.getTabName(tabIndex), model.getTabImage(tabIndex));
		// initialize tab
		initTab(tab, tabIndex);
		// add default tab mouse listener
		tab.addMouseListener(new TabMouseListener(this));
		// add this as shutter label listener
		tab.addShutterLabelListener(this);
		// add all listeners
		addAllListenersToATab(tab);
		// add to array
		tabControls.add(tabIndex, tab);		
			
		// create a scroll list corresponding to tab
		ScrollList sl = new ScrollList(this, SWT.NONE);
		// initialize scroll list
		initScrollList(sl, m.getTabModel(tabIndex), tab);
		// add scroll list to array
		scrollLists.add(tabIndex, sl);
		
		// adjust SortData index of tabs which are after inserted tab
		int size = tabControls.size();
		for(int i = tabIndex + 1; i < size; i++)
			((SortData)getTabControl(i).getLayoutData()).index++;
		
		// adjust current tab index because tabIndex may smaller than current tab
		if(tabIndex <= currentTab)
			currentTab++;
			
		// жǷӦù˵tab
		//     ѡtabڱ˵Ʒ֣Ǿ͹֮
		//     tabڱ˵ķΧôӦÿܲ
		//     ǰtabֻѡһ
		if(tabFilter != null) {
			if(tabFilter.match(model.getTab(tabIndex)))
				hideTab(tabIndex, true);
			else if(getVisibleTabCount() == 1)
				setCurrentTab(tabIndex);			
		}
			
		// relayout
		this.layout();
		
		// call model listeners
		int count = modelListeners.size();
		for(int i=0;i<count;i++)
			((ShutterModelListener)modelListeners.get(i)).modelAdded(e);
	}

	/* (non-Javadoc)
	 * @see edu.tsinghua.swt.events.ModelListener#modelRemoved(edu.tsinghua.swt.events.ModelEvent)
	 */
	public void modelRemoved(ModelEvent e) {
		ShutterModelEvent event = (ShutterModelEvent)e;
		int tabIndex = event.tabIndex;
		ShutterModel m = (ShutterModel)event.getSource();
		
		// if this tab is current tab, re set the tab above this as current tab
		// if this tab is the top tab, re set the tab below this as current tab
		if(tabIndex==currentTab) {
			int candidate = chooseCurrentTab(tabIndex);
			if(candidate != -1)
				setCurrentTab(candidate);
			else  // this is the last tab, so currentTab is nothing, set it as -1
				currentTab = -1;
		}
		// adjust SortData index of tabs which are after the tab deleted
		int size = tabControls.size();
		for(int i = tabIndex + 1; i < size; i++)
			((SortData)getTabControl(i).getLayoutData()).index--;
		// dispose tab and scroll list
		CoolButton tab = (CoolButton)tabControls.remove(tabIndex);
		ScrollList sl = (ScrollList)scrollLists.remove(tabIndex);
		sl.dispose();
		tab.dispose();
		// notice here we need substract 1 from current tab when
		//    the tabIndex < current tab, because we had changed
		//    current tab, now tabIndex can only > or < current 
		//    tab, equal is impossible
		if(tabIndex < currentTab) currentTab--;
		// relayout
		this.layout();
		
		// call model listeners
		int count = modelListeners.size();
		for(int i=0;i<count;i++)
			((ShutterModelListener)modelListeners.get(i)).modelRemoved(e);
	}

	/* (non-Javadoc)
	 * @see edu.tsinghua.swt.events.ShutterModelListener#itemAdded(edu.tsinghua.swt.events.ShutterModelEvent)
	 */
	public void itemAdded(ShutterModelEvent e) {
		// call model listeners
		int count = modelListeners.size();
		for(int i=0;i<count;i++)
			((ShutterModelListener)modelListeners.get(i)).itemAdded(e);
	}

	/* (non-Javadoc)
	 * @see edu.tsinghua.swt.events.ShutterModelListener#itemRemoved(edu.tsinghua.swt.events.ShutterModelEvent)
	 */
	public void itemRemoved(ShutterModelEvent e) {
		// call model listeners
		int count = modelListeners.size();
		for(int i=0;i<count;i++)
			((ShutterModelListener)modelListeners.get(i)).itemRemoved(e);
	}

	/* (non-Javadoc)
	 * @see edu.tsinghua.swt.events.PropertyListener#propertyAdded(edu.tsinghua.swt.events.PropertyEvent)
	 */
	public void propertyAdded(PropertyEvent e) {
		int index = model.indexOf(e.getSource());
		// -1˵tabitem
		if(index != -1) {
			if(tabFilter != null)
				if(tabFilter.match(model.getTab(index)))
					hideTab(index, true);
			// relayout if current layout is in sort mode
			if(((SortLayout)this.getLayout()).sort)
				this.layout();			
		}
		// call property listeners
		int count = propertyListeners.size();
		for(int i=0;i<count;i++)
			((PropertyListener)propertyListeners.get(i)).propertyAdded(e);
	}

	/* (non-Javadoc)
	 * @see edu.tsinghua.swt.events.PropertyListener#propertyRemoved(edu.tsinghua.swt.events.PropertyEvent)
	 */
	public void propertyRemoved(PropertyEvent e) {
		int index = model.indexOf(e.getSource());
		// -1˵tabitem
		if(index != -1) {
			if(tabFilter != null)
				if(tabFilter.match(model.getTab(index)))
					hideTab(index, true);
			// relayout if current layout is in sort mode
			if(((SortLayout)this.getLayout()).sort)
				this.layout();						
		}
		// call property listeners
		int count = propertyListeners.size();
		for(int i=0;i<count;i++)
			((PropertyListener)propertyListeners.get(i)).propertyRemoved(e);
	}

	/* (non-Javadoc)
	 * @see edu.tsinghua.swt.events.PropertyListener#propertyModified(edu.tsinghua.swt.events.PropertyEvent)
	 */
	public void propertyModified(PropertyEvent e) {
		int index = model.indexOf(e.getSource());
		// -1˵tabitem
		if(index != -1) {
			// жǷӦù˵tab
			if(tabFilter != null)
				if(tabFilter.match(model.getTab(index)))
					hideTab(index, true);
			// relayout if current layout is in sort mode
			if(((SortLayout)this.getLayout()).sort)
				this.layout();			
		}
		// call property listeners
		int count = propertyListeners.size();
		for(int i=0;i<count;i++)
			((PropertyListener)propertyListeners.get(i)).propertyModified(e);
	}

	/* (non-Javadoc)
	 * @see edu.tsinghua.swt.events.CoolButtonListener#textChanged(edu.tsinghua.swt.events.CoolButtonEvent)
	 */
	public void textChanged(CoolButtonEvent e) {
		// modify layout sort field
		CoolButton tab = (CoolButton)e.getSource();
		((SortData)tab.getLayoutData()).key = e.newText;
		// modify model
		model.setTabName(indexOf(tab), e.newText);
		// relayout if current layout is in sort mode
		if(((SortLayout)this.getLayout()).sort)
			this.layout();
	}
	
	/**
	 * ƶtabʾֵɫ
	 * @param color ɫ
	 */
	public void setTabMouseTrackColor(Color color) {
		if(color == null) return;
		this.tabMouseTrackColor = color;
		int size = tabControls.size();
		for(int i = 0; i < size; i++)
			((CoolButton)tabControls.get(i)).setMouseTrackColor(tabMouseTrackColor);
	}
	
	/**
	 * ƶitemʱʾֵɫ
	 * @param color ɫ
	 */
	public void setItemMouseTrackColor(Color color) {
		if(color == null) return;
		this.itemMouseTrackColor = color;
		int size = scrollLists.size();
		for(int i = 0; i < size; i++)
			((ScrollList)scrollLists.get(i)).setMouseTrackColor(itemMouseTrackColor);
	}
	
	/**
	 * öTabĶTabmodelҲGeneralNode
	 * @param sorter 
	 */
	public void setTabSorter(Comparator sorter) {
		if(sorter == null) return;
		((SortLayout)this.getLayout()).keyComparator = sorter;
	}
	
	/**
	 * öItemĶItemmodelҲGeneralNode
	 * @param sorter 
	 */
	public void setItemSorter(Comparator sorter) {
		this.itemSorter = sorter;
		int size = scrollLists.size();
		for(int i = 0; i < size; i++)
			((ScrollList)scrollLists.get(i)).setSorter(itemSorter);
	}
	
	/**
	 * öItemĶItemmodelҲGeneralNode
	 * ֻtabIndexָtab
	 * @param tabIndex tab
	 * @param sorter 
	 */
	public void setItemSorter(int tabIndex, Comparator sorter) {
		if(tabIndex < 0 || tabIndex >= scrollLists.size()) return;
		((ScrollList)scrollLists.get(tabIndex)).setSorter(sorter);
	}
	
	/**
	 * @param itemExtra The itemExtra to set.
	 */
	public void setItemExtra(IExtraWork extra) {
		this.itemExtra = extra;
		int size = scrollLists.size();
		for(int i = 0; i < size; i++)
			((ScrollList)scrollLists.get(i)).setExtraWork(itemExtra);
	}
	
	/**
	 * @param tabExtra The tabExtra to set.
	 */
	public void setTabExtra(IExtraWork extra) {
		this.tabExtra = extra;
	}
	
	/**
	 * һTab
	 * @param tabIndex Ҫtabindex
	 * @param layout trueʾlayoutfalseʾlayoutһض
	 * 		   Tabʱһlayout
	 */
	public void hideTab(int tabIndex, boolean layout) {
		CoolButton tab = getTabControl(tabIndex);
		if(tab == null) return;
		tab.setVisible(false);
		SortData sd = (SortData)tab.getLayoutData();
		sd.heightHint = 0;
		// tabIndexǵǰTabҪ赱ǰTab
		if(tabIndex == currentTab) {
			int candidate = chooseCurrentTab(tabIndex);
			if(candidate != -1)
				setCurrentTab(candidate);
			else {
				ScrollList sl = (ScrollList)scrollLists.get(tabIndex);
				sl.setVisible(false);
				sd = (SortData)sl.getLayoutData();
				sd.heightHint = 0;
				sd.grabExcessVerticalSpace = false;
				sd.verticalAlignment = SortData.BEGINNING;
				if(layout) this.layout();
			}
		} else { // ǣ򵥵layout
			if(layout) this.layout();
		}
	}
	
	/**
	 * ʾһtabtabǿ״̬Ч
	 * @param tabIndex Ҫʾtab index
	 * @param layout trueʾlayoutfalseʾlayoutһض
	 * 		   Tabʱһlayout
	 */
	public void showTab(int tabIndex, boolean layout) {
		CoolButton tab = getTabControl(tabIndex);
		if(tab == null) return;
		if(tab.getVisible())
			return;
		else {
			tab.setVisible(true);
			SortData sd = (SortData)tab.getLayoutData();
			sd.heightHint = SWT.DEFAULT;
			if(getVisibleTabCount() == 1) {
				/*
				 *  tabΪ1עvisibleΪtrue˵Ŀǰûеǰtab
				 * ҪtabΪǰtab 
				 */
				setCurrentTab(tabIndex);
			} else {
				/* 
				 * tabΪ1˵˵ǰtabеģΪtabIndexָtabӣҲͲ
				 * ΪǰtabҪsetCurrentTab
				 */
				ScrollList sl = (ScrollList)scrollLists.get(tabIndex);
				sl.setVisible(true);
				if(layout)
					this.layout();
			}
		}
	}
	
	// tabIndexָtabǵǰtabôtabɾ˻߱ˣѡһtabΪǰtab
	private int chooseCurrentTab(int tabIndex) {
		int size = tabControls.size();
		int candidate = tabIndex;
		int direction = -1;
		while(--size > 0) {
			// tabIndexϿʼѡindexʼС0ٴtabIndexѡ
			candidate += direction;
			if(candidate < 0) {
				candidate = tabIndex + 1;
				direction = 1;
			}
			// visibleΪtrue򷵻
			CoolButton tab = (CoolButton)tabControls.get(candidate);
			if(tab.getVisible())
				return candidate;
		}
		return -1;
	}
	
	// õǰӵtab
	private int getVisibleTabCount() {
		Iterator iter = tabControls.iterator();
		int count = 0;
		while(iter.hasNext()) {
			CoolButton tab = (CoolButton)iter.next();
			if(tab.getVisible())
				count++;
		}
		return count;
	}
	
	/**
	 * facadescroll listhideItemʵ
	 * @param tabIndex
	 * @param itemIndex
	 * @param layout trueʾlayoutfalseʾlayoutһض
	 * 		   Itemʱһlayout
	 */
	public void hideItem(int tabIndex, int itemIndex, boolean layout) {
		if(tabIndex < 0 || tabIndex >= tabControls.size()) return;
		((ScrollList)scrollLists.get(tabIndex)).hideItem(itemIndex, layout);
	}
	
	/**
	 * facadescroll listshowItemʵ
	 * @param tabIndex
	 * @param itemIndex
	 * @param layout trueʾlayoutfalseʾlayoutһض
	 * 		   Itemʱһlayout
	 */
	public void showItem(int tabIndex, int itemIndex, boolean layout) {
		if(tabIndex < 0 || tabIndex >= tabControls.size()) return;
		((ScrollList)scrollLists.get(tabIndex)).showItem(itemIndex, layout);
	}
	
	/**
	 * itemûṩһMatcherʵ֣ڷtrueֵitemʾ
	 * @param filter
	 */
	public void filterTab(IMatcher filter) {
		// filter
		this.tabFilter = filter;
		// tab
		int size = tabControls.size();
		for(int i = 0; i < size; i++) 
			getTabControl(i).setVisible(true);
		// طtab
		int[] matches = model.findTabIndex(filter);
		if(matches == null) return;
		for(int i = 0; i < matches.length; i++)
			hideTab(matches[i], true);
	}	
	
	/**
	 * facadeʵʵscroll listķ
	 * @param filter IMatcher
	 */
	public void filterItem(IMatcher filter) {
		// filter
		this.itemFilter = filter;
		// scroll listķ
		int size = tabControls.size();
		for(int i = 0; i < size; i++)
			((ScrollList)scrollLists.get(i)).filter(filter);
	}
	
	/**
	 * @return Returns the showSmall.
	 */
	public boolean isShowSmallImage() {
		return showSmall;
	}
	
	/**
	 * @return Returns the showText.
	 */
	public boolean isShowText() {
		return showText;
	}
	
	/**
	 * @param contentAlignment The contentAlignment to set.
	 */
	public void setContentAlignment(int contentAlignment) {
		this.contentAlignment = contentAlignment;
        Iterator iter = scrollLists.iterator();
        while(iter.hasNext())
            ((ScrollList)iter.next()).setContentAlignment(contentAlignment);
	}
	
	/**
	 * ˢtabĲ
	 * @param tabIndex tabindex
	 */
	public void refreshTabView(int tabIndex) {
		if(tabIndex < 0 || tabIndex >= scrollLists.size()) return;
		((ScrollList)scrollLists.get(tabIndex)).refreshView();
	}
}
